<?php
/*-----------------------------------------------\
| 												 |
|  Author:       Andrey Brykin (Drunya)          |
|  Version:      1.5.2                           |
|  Project:      CMS                             |
|  package       CMS Fapos                       |
|  subpackege    Forum Module                    |
|  copyright     ©Andrey Brykin 2010-2011        |
\-----------------------------------------------*/

/*-----------------------------------------------\
| 												 |
|  any partial or not partial extension          |
|  CMS Fapos,without the consent of the          |
|  author, is illegal                            |
|------------------------------------------------|
|  Любое распространение                         |
|  CMS Fapos или ее частей,                      |
|  без согласия автора, является не законным     |
\-----------------------------------------------*/


/**
* forum functionaly
*
* @author      Andrey Brykin 
* @package     CMS Fapos
* @subpackage  Forum module
* @link        http://cms.develdo.com
*/
Class ForumModule extends Module {

	/**
	* @module_title  title of module
	*/
	public $module_title = 'Форум';
	/**
	* @template  layout for module
	*/
	public $template = 'forum';
	/**
	* @module module indentifier
	*/
	public $module = 'forum';
	


	/**
	* @return main forum page content
	*/
	public function index($cat_id = null) 
	{
		//turn access
		$this->ACL->turn(array('forum', 'view_forums_list'));
		
		if ($this->cached && $this->Cache->check($this->cacheKey)) {
			$html = $this->Cache->read($this->cacheKey);
		} else {
			$this->page_title = h(Config::read('title', 'forum')) . __('Forums list');
			$html = '';
			$conditions = array();
			if (!empty($cat_id) && is_numeric($cat_id)) {
				$cat_id = (int)$cat_id;
				if ($cat_id > 0) {
					$conditions['id'] = $cat_id;
				}
			}
			
			//get forums categories records
			$res = $this->DB->select('forum_cat', DB_ALL, array('cond' => $conditions, 'order' => 'previev_id'));
			if (count($res) > 0) {
			
				$conditions = (!empty($conditions)) ? array('`a`.`in_cat`' => $cat_id) : array();
				$conditions[] = array("`a`.`parent_forum_id` IS NULL OR `a`.`parent_forum_id` = '0'");
				$_forums = $this->DB->select('forums', DB_ALL, array(
					'joins' => array(
						array(
							'table' => 'themes',
							'alias' => 'b',
							'cond' => '`a`.`last_theme_id` = `b`.`id`',
							'type' => 'LEFT',
						),
						array(
							'table' => 'users',
							'alias' => 'c',
							'cond' => '`c`.`id` = `b`.`id_last_author`',
							'type' => 'LEFT',
						),
					),
					'alias' => 'a',
					'cond' => $conditions,
					'fields' => array('a.*', 'b.`id_last_author`', 'b.`title` as last_theme_title',
					'b.`last_post` as last_post_date', 'c.`name` AS last_author'),
					'order' => '`a`.`pos`',
				));
				//get sub forums
				$_subforums = $this->DB->select('forums', DB_ALL, array(
					'cond' => array('`parent_forum_id` IS NOT NULL'),
					'order' => 'pos',
				));
				
				
				//sort forums and subforums
				//after this we will be have $categories array with all cats, forum and subforums
				$forums = array();
				$categories = array();
				if (count($_forums) > 0) {
					foreach ($_forums as $forum) {
						if (count($_subforums) > 0) {
							foreach ($_subforums as $subforum) {
								if ($subforum['parent_forum_id'] == $forum['id']) {
									$forum['subforums'][] = $subforum;
								}
							}
						}
						$forums[$forum['in_cat']][] = $forum;
					}
					unset($_subforums); //clean memory
				}
				
				
				foreach ($res as $category) {
					$categories[$category['id']] = $category;
					$categories[$category['id']]['forums'] = array();
					if (array_key_exists($category['id'], $forums)) {
						$categories[$category['id']]['forums'] = $forums[$category['id']];
						unset($forums[$category['id']]); //clean memory
					} else {
						unset($categories[$category['id']]); //we needen't empty categories
					}
				}

			
				foreach ($categories as $cats) {
					$c_markets = array();
					$htmlcat = $this->Parser->getAndParse('vievcat.html');
					$c_markets['{CAT_NAME}'] = h($cats['title']);
					$c_markets['{CAT_URL}'] = get_url('/forum/index/' . $cats['id']);
						
						
					if (count($cats['forums']) > 0) {
						$info = '';
						foreach ($cats['forums'] as $forum) {
							$f_markets = array();
							$subforums = (!empty($forum['subforums']) && is_array($forum['subforums'])) 
								? $forum['subforums'] : array();
							
						
							//forming forums
							$htmlforum = $this->_parseForumTable($forum);
							
							
							//subforums
							$subforums_content = '';
							if (count($subforums) > 0) {
								$subforums_content .= __('Subforums') . '<ul class="subforums">';
								foreach($subforums as $subforum) {
									$subforums_content .= '<li>' . get_link(h($subforum['title']), 
									'/forum/view_forum/' . $subforum['id'], array('class' => 'subforum')) . '</li>';
								}
								$subforums_content .= '</ul>';
							}
							
							
							$f_markets['{SUBFORUMS}'] = $subforums_content;
							$info .= $this->_replaceMarkets($f_markets, $htmlforum);
							
						}
					} else {
						$info = __('Subforum is empty');
					}
					
					
					$c_markets['{FORUMS}'] = $info;
					$html .= $this->_replaceMarkets($c_markets, $htmlcat);
				}
				//write to cache ( only if records detected )
				if ($this->cached)
					$this->Cache->write($html, $this->cacheKey, $this->cacheTags);		
			} else {
				$html = __('No categories') . "\n";
			}
		}
		
		// navigation block
		$markets = array();
		$markets['{NAVIGATION}'] = get_link(__('Home'), '/') . __('Separator') 
		. get_link(__('Forums list'), '/forum/') . "\n";
		$markets['{PAGINATION}'] = '';
		$markets['{ADD_LINK}'] = '';
		$markets['{META}'] = '';
		
		$html = $html . $this->_get_stat();
		$this->_globalize($markets);
		return $this->_view($html);
	}


	
	/**
	 * @param array $forum
	 * @retrun string HTML forum table wiht replaced markets
	 */
	private function _parseForumTable($forum) 
	{
		// Summ posts and themes
		if (isset($forum['subforums']) && count($forum['subforums']) > 0) {
			foreach ($forum['subforums'] as $subforum) {
				$forum['posts'] += $subforum['posts'];
				$forum['themes'] += $subforum['themes'];
			}
			unset($forum['subforums']);
		}
		
	
		$markets = array();
		$htmlforum = $this->Parser->getAndParse('vievforum.html');
		$markets['{FORUM_URL}'] = get_url('/forum/view_forum/' . $forum['id']);
		
		foreach($forum as $key => $value) {
			$markets['{' . strtoupper($key) . '}'] = h($value);
		}
		//count themes and posts of forum
		$markets['{COUNT_THEMES}'] = $forum['themes'];
		$markets['{COUNT_POSTS}'] = $forum['posts'];
		

		//выводим название темы в которой было добавлено последнее сообщение и дату его добавления
		if (empty($forum['last_theme_id']) || empty($forum['id_last_author']) 
		|| empty($forum['last_author']) || empty($forum['last_post_date'])) {
			$last_post = __('No posts');
			
		} else {
			$last_post_title = (mb_strlen($forum['last_theme_title']) > 30) 
			? mb_substr($forum['last_theme_title'], 0, 30) . '...' : $forum['last_theme_title'];
			$last_post = $forum['last_post_date'] . '<br>' . get_link(h($last_post_title), 
				'/forum/view_theme/' . $forum['last_theme_id'] . '?page=999', 
				array('title' => __('To last post')))
			    . __('Post author') . get_link(h($forum['last_author']), 
				getProfileUrl($forum['id_last_author']), array('title' => __('To profile')));
		}
		$markets['{LAST_POST}'] = $last_post;
		

		
		// Ссылка "Править клубы,"
		$admin_bar = '';
		if ($this->ACL->turn(array('forum', 'replace_forums'), false)) {
			$admin_bar .= get_link(get_img('/sys/img/up_arrow_16x16.png'), 'forum/forum_up/' . $forum['id'])
				. '&nbsp;' . get_link(get_img('/sys/img/down_arrow_16x16.png'), 'forum/forum_down/' . $forum['id']) . '&nbsp;';
		}
		if ($this->ACL->turn(array('forum', 'edit_forums'), false)) {
			$admin_bar .= get_link(get_img('/sys/img/edit_16x16.png'), 'forum/edit_forum_form/' . $forum['id']) . '&nbsp;';
		}
		if ($this->ACL->turn(array('forum', 'delete_forums'), false)) {
			$admin_bar .= get_link(get_img('/sys/img/delete_16x16.png'), 'forum/delete_forum/' . $forum['id'],
			array('onClick' => "return confirm('" . __('Are you sure') . "')")) . '&nbsp;';
		}
		$markets['{ADMIN_BAR}'] = $admin_bar;
		
		
		/* forum icon */
		$forum_icon = get_url('/sys/img/guest.gif');
		if (file_exists(R . 'sys/img/forum_icon_' . $forum['id'] . '.jpg')) {
			$forum_icon = get_url('/sys/img/forum_icon_' . $forum['id'] . '.jpg');
		}
		$markets['{FORUM_ICON_URL}'] = $forum_icon;
		$htmlforum = $this->_replaceMarkets($markets, $htmlforum);
		return $htmlforum;
	}
	
	


	/**
	 * View threads list (forum)
	 */
	public function view_forum($id_forum = null) 
	{
		//turn access
		$this->ACL->turn(array('forum', 'view_forums'));
		$id_forum = intval($id_forum);
		if (empty($id_forum) || $id_forum < 1) redirect('/forum/');


		
		//are we have cache?
		if ($this->cached && $this->Cache->check($this->cacheKey)) {
			$html = $this->Cache->read($this->cacheKey);
		} else {
			$markets = array();
		
			// Получаем информацию о клубе
			$query = $this->DB->select('forums', DB_FIRST, array('cond' => array('id' => $id_forum)));
			if (empty($query)) {
				return showInfoMessage(__('Can not find forum'), '/forum/');
			}
			
			
			// Check access to this forum. May be locked by pass or posts count
			$this->__checkForumAccess($query[0]);
			
			$forum = $query[0];
			$this->page_title = $this->page_title . ' - ' . h($forum['title']);

			// Выводим название клуба
			$htm = $this->Parser->getAndParse('forumtable.html');
			$markets['{FORUM_NAME}'] = h($forum['title']);
			$html = '';

			// reply link
			$addLink = (isset( $_SESSION['user'])) ? get_link(get_img('/sys/img/add_theme_button.png', 
			array('alt' => __('New topic'))), '/forum/add_theme_form/' . $id_forum) : '';
			// count themes for page nav
			$total = $this->DB->select('themes', DB_COUNT, array('cond' => array('id_forum' => $id_forum)));
			
			
			// page nav
			$cntPages = ceil( $total / Config::read('themes_per_page', 'forum') );
			if (isset($_GET['page'])) {
				$page = (int)$_GET['page'];
				if ($page < 1) $page = 1;
			} else {
				$page = 1;
			}
			if ($page > $cntPages) $page = $cntPages;
			if ($cntPages > 1) {
				$pages = pagination( $page, $cntPages, '/forum/view_forum/' . $id_forum );
				$this->page_title .= ' (' . $page . ')';
			} else {
				$pages = '&nbsp;';
			}
			
			
			// Nav block
			$n_markets = array();
			$n_markets['{NAVIGATION}'] = get_link(__('Forums list'), '/forum/') . __('Separator') 
			. get_link(h($forum['title']), '/forum/view_forum/' . $id_forum);
			$n_markets['{PAGINATION}'] = $pages;
			$n_markets['{ADD_LINK}'] = '';
			$n_markets['{META}'] = $addLink;
			$this->_globalize($n_markets);

			
			//get records
			$res = $this->DB->select('themes', DB_ALL, array(
				'joins' => array(
					array(
						'table' => 'users',
						'alias' => 'b',
						'cond' => '`b`.`id` = `a`.`id_author`',
						'type' => 'LEFT',
					),
					array(
						'table' => 'users',
						'alias' => 'c',
						'cond' => '`c`.`id` = `a`.`id_last_author`',
						'type' => 'LEFT',
					),
				),
				'cond' => array('`a`.`id_forum`' => $id_forum),
				'fields' => array('a.*', 'b.`name` AS author', 'c.`name` AS last_author'),
				'alias' => 'a',
				'order' => 'important DESC, last_post DESC',
				'page' => $page,
				'limit' => Config::read('themes_per_page', 'forum'),
			));		
				
			//subforums
			$subforums_content = '';
			$subforums = $this->DB->select('forums', DB_ALL, array(
				'joins' => array(
					array(
						'table' => 'themes',
						'alias' => 'b',
						'type' => 'LEFT',
						'cond' => '`a`.`last_theme_id` = `b`.`id`',
					),
					array(
						'table' => 'users',
						'alias' => 'c',
						'type' => 'LEFT',
						'cond' => '`c`.`id` = `b`.`id_last_author`',
					),
				),
				'alias' => 'a',
				'order' => '`a`.`pos` ASC',
				'cond' => array('`a`.`parent_forum_id`' => $id_forum),
				'fields' => array('a.*', 'b.`id_last_author`', 'b.`title` as last_theme_title',
				'b.`last_post` as last_post_date', 'c.`name` AS last_author'),
			));	
			if (count($subforums) > 0) {
				$subforums_templ = $this->Parser->getAndParse('vievcat.html');
				foreach($subforums as $subforum) {
					$subforums_content .= $this->_parseForumTable($subforum);
				}
				$s_markets = array();
				$s_markets['{FORUMS}'] = $subforums_content;
				$s_markets['{CAT_NAME}'] = __('Subforums title');
				$s_markets['{SUBFORUMS}'] = '';
				$subforums_content = $this->_replaceMarkets($s_markets, $subforums_templ);
			}
			
			
			
			$cnt_themes_here = count($res);
			if ($cnt_themes_here > 0) {
				$info = null;
				foreach ($res as $theme) {
					$info .= $this->__parseThemeTable($theme);
					$htmltheme = $info;
					
					//set cache tags
					$this->setCacheTag(array(
						'theme_id_' . $theme['id'],
					));
				}			
				$this->setCacheTag(array(
					'forum_id_' . $id_forum,
				));
				
				
			} else {
				$htmltheme = __('No topics');
				$this->cached = false;
			}	
			
				$markets['{COUNT_THEMES_HERE}'] = $cnt_themes_here;
				$markets['{COUNT_THEMES}'] = $total;
				$markets['{THEMES}'] = $htmltheme;
				$markets['{SUBFORUMS}'] = $subforums_content;
				$html = $this->_replaceMarkets($markets, $htm);

			//write cache
			if ($this->cached)
				$this->Cache->write($html, $this->cacheKey, $this->cacheTags);
		}
		
		
		//who is here
		$who = array();	
		if (!file_exists(R . 'sys/logs/forum')) mkdir(R . 'sys/logs/forum', 0777, true);
		if (file_exists(R . 'sys/logs/forum/' . $id_forum . '.dat')) {
			$who = unserialize(file_get_contents(R . 'sys/logs/forum/' . $id_forum . '.dat'));
		}
		if (isset($_SESSION['user'])) {
			if (!isset($who[$_SESSION['user']['id']])) {
				$who[$_SESSION['user']['id']]['profile_link'] = get_link(h($_SESSION['user']['name']), '/users/info/' . $_SESSION['user']['id']);
				$who[$_SESSION['user']['id']]['expire'] = time() + 1000;
			}
		}
		
		$who_is_here = '';
		foreach ($who as $key => $val) {
			if ($val['expire'] < time()) {
				unset($who[$key]);
				continue;
			}
			$who_is_here .= $val['profile_link'] . ', ';
		}
		file_put_contents(R . 'sys/logs/forum/' . $id_forum . '.dat', serialize($who));
		$html = str_replace('{WHO_IS_HERE}', substr($who_is_here, 0, -2), $html);

		return $this->_view($html);
	}


	
	/**
	 * Check access to this forum. 
	 * May be locked by pass or posts count
	 *
	 * @param array $forum
	 */
	private function __checkForumAccess($forum) {
		if (empty($forum['lock_passwd']) && empty($forum['lock_posts'])) return;
		
		
		if (!empty($forum['lock_passwd'])) {
			if (isset($_SESSION['access_forum_' . $forum['id']])) {
				return;
				
			// if we have two generally meet one of them
			} else if (!empty($forum['lock_posts']) && 
			(isset($_SESSION['user']['posts']) && $_SESSION['user']['posts'] >= $forum['lock_posts'])) {
				return;
				
			// Check sended password
			} else if (isset($_POST['forum_lock_pass'])) {
				if ($_POST['forum_lock_pass'] == $forum['lock_passwd']) {
					$_SESSION['access_forum_' . $forum['id']] = true;
					return;
				}
				showInfoMessage(__('Wrong pass for forum'), '/forum/');
			} else {
				echo $this->Parser->getAndParse('forum_passwd_form.html');
				die();
			}
			
			
		// For lock by posts count
		} else if (!empty($forum['lock_posts'])) {
			if (isset($_SESSION['user']['posts']) && $_SESSION['user']['posts'] >= $forum['lock_posts']) {
				return;
			}
			showInfoMessage(sprintf(__('locked forum by posts'), $forum['lock_posts']), '/forum/');
		}
	}
	
	

	/**
	 * @param array $theme
	 * @param string $template
	 * If $template = FALSE, use default template file
	 * @retrun string HTML theme table with replaced markets
	 */
	private function __parseThemeTable($theme, $template = false) 
	{
		$htmltheme = null;
		$markets = array();
		
		//ICONS
		$themeicon = $this->__getThemeIcon($theme); 

		
		if (!$template)
			$htmlfile = $this->Parser->getAndParse('vievthemes.html');
		else 
			$htmlfile = $template;
			
		$markets['{THEME_URL}'] = get_url('/forum/view_theme/' . $theme['id']);

		
		//ADMINBAR 
		$adminbar = '';
		if ($this->ACL->turn(array('forum', 'edit_themes'), false) 
		|| (!empty($_SESSION['user']['id']) && $theme['id_author'] == $_SESSION['user']['id'] 
		&& $this->ACL->turn(array('forum', 'edit_mine_themes'), false))) {
			$adminbar .= get_link(get_img('/sys/img/edit_16x16.png', array('alt' => __('Edit'), 'title' => __('Edit'))),
				'/forum/edit_theme_form/' . $theme['id']);
		}
		if ($this->ACL->turn(array('forum', 'close_themes'), false)) {
			if ( $theme['locked'] == 0 ) { // заблокировать тему
				$adminbar .= get_link(get_img('/sys/img/lock_16x16.png', array('alt' => __('Lock'), 'title' => __('Lock'))),
					'/forum/lock_theme/' . $theme['id']);
			} else { // разблокировать тему
				$adminbar .= get_link(get_img('/sys/img/unlock_16x16.png', array('alt' => __('Unlock'), 'title' => __('Unlock'))),
					'/forum/unlock_theme/' . $theme['id']);
			}
		}
		
		
		if ($this->ACL->turn(array('forum', 'important_themes'), false)) {
			if ($theme['important'] == 1) {
				$adminbar .= get_link(get_img('/sys/img/unpush.png', array('alt' => __('Important'), 'title' => __('Important'))),
					'/forum/unimportant/' . $theme['id']);
			} else {
				$adminbar .= get_link(get_img('/sys/img/push.png', array('alt' => __('Not important'), 'title' => __('Not important'))),
					'/forum/important/' . $theme['id']);
			}
		}
		
		
		if ($this->ACL->turn(array('forum', 'delete_themes'), false) 
		|| (!empty($_SESSION['user']['id']) && $theme['id_author'] == $_SESSION['user']['id'] 
		&& $this->ACL->turn(array('forum', 'delete_mine_themes'), false))) {
			$adminbar .= get_link(get_img('/sys/img/delete_16x16.png', array('alt' => __('Delete'), 'title' => __('Delete'))),
				'/forum/delete_theme/' . $theme['id'], array('onClick' => "return confirm('" . __('Are you sure') . "')"));
		}
		$markets['{ADMINBAR}'] = $adminbar;
		
		
		//USER PROFILE
		$markets['{AUTHOR}'] = get_link(h($theme['author']), getProfileUrl($theme['id_author'])); 
		
		foreach($theme as $key => $value) {
			if ($key != 'title' && $key != 'author') $markets['{'.strtoupper($key).'}'] = h($value); 
		}	
		
		//Выводим последнего автора и ссылку на последнюу страницу
		$last_user = get_link(h($theme['last_author']), getProfileUrl($theme['id_last_author']));
		$last_page = get_link(__('To last'), '/forum/view_theme/' . $theme['id'] . '&page=99999');
		
		//NEAR PAGES
		$near_pages = '';
		if (($theme['posts'] + 1) > Config::read('posts_per_page', 'forum')) {
			$cnt_near_pages = ceil(($theme['posts'] + 1) / Config::read('posts_per_page', 'forum'));
			if ($cnt_near_pages > 1) {
				$near_pages .= '&nbsp;(';
				for ($n = 1; $n < ($cnt_near_pages + 1); $n++) { 
					if ($cnt_near_pages > 5 && $n > 3) {
						$near_pages .= '...&nbsp;' . get_link(($cnt_near_pages - 1), '/forum/view_theme/' . $theme['id'] . '?page=' 
									. ($cnt_near_pages - 1)) . '&nbsp;' . get_link($cnt_near_pages, '/forum/view_theme/' 
									. $theme['id'] . '?page=' . $cnt_near_pages) . '&nbsp;';				
						break;
					} else {
						if ($n > 5) break;
						$near_pages .= get_link($n, '/forum/view_theme/' . $theme['id'] . '?page=' . $n) . '&nbsp;';
					}
				}
				$near_pages .= ')';
			}
		}
		
		$theme_title = h($theme['title']);
		$theme_description = h($theme['description']);
		$important = ($theme['important'] == 1) ? __('Important2') : '';
		$markets['{IMPORTANTLY}'] = $important;
		$markets['{LAST_PAGE}'] = $last_page;
		$markets['{LAST_USER}'] = $last_user;
		$markets['{COUNT_POSTS}'] = $theme['posts'];
		$markets['{THEMEICON}'] = $themeicon;
		$markets['{NEAR_PAGES}'] = $near_pages;
		$markets['{TITLE}'] = $theme_title;
		$markets['{DESCRIPTION}'] = $theme_description;
		$htmltheme = $this->_replaceMarkets($markets, $htmlfile);

		return $htmltheme;
	}
	
	

	/**
	 * Return theme icon
	 *
	 * @param array $theme
	 * @return string img HTML tag with URL to needed icon
	 */
	private function __getThemeIcon($theme) {
		if (isset($_SESSION['user'])) { // это для зарегистрированного пользователя
			// Если есть новые сообщения (посты) - только для зарегистрированных пользователей
			if ( isset( $_SESSION['newThemes'] ) and in_array( $theme['id'], $_SESSION['newThemes'] ) ) {
				if ( $theme['locked'] == 0 ) // тема открыта
					$themeicon = get_img('/sys/img/folder_new.gif', array('width' => '19', 'height' => '18', 
						'alt' => __('New posts'), 'title' => __('New posts')));
				else // тема закрыта
					$themeicon = get_img('/sys/img/folder_lock_new.gif', array('width' => '19', 'height' => '18', 
						'alt' => __('New posts'), 'title' => __('New posts')));
						
						
			} else {
				if ( $theme['locked'] == 0 ) // тема открыта
					$themeicon = get_img('/sys/img/folder.gif', array('width' => '19', 'height' => '18', 
						'alt' => __('No new posts'), 'title' => __('No new posts')));
				else // тема закрыта
					$themeicon = get_img('/sys/img/folder_lock.gif', array('width' => '19', 'height' => '18', 
						'alt' => __('No new posts'), 'title' => __('No new posts')));
			}
			
			
		} else { // это для не зарегистрированного пользователя
			if ( $theme['locked'] == 0 ) // тема открыта
				$themeicon = get_img('/sys/img/folder.gif', array('width' => '19', 'height' => '18'));
			else // тема закрыта
				$themeicon = get_img('/sys/img/folder_lock.gif', array('width' => '19', 'height' => '18'));
		}
		
		return $themeicon;
	}
	

	/**
	 * Return posts list
	 */
	public function view_theme($id_theme = null) {	
		//turn access
		$this->ACL->turn(array('forum', 'view_themes'));
		$id_theme = (int)$id_theme;
		if (empty($id_theme) || $id_theme < 1) redirect('/forum/');

		
		// Получаем информацию о форуме - это нужно для построения панели навигации
		$forum = $this->DB->select('forums', DB_FIRST, array(
			'joins' => array(
				array(
					'table' => 'themes',
					'alias' => 'b',
					'type' => 'LEFT',
					'cond' => array('`b`.`id_forum` = `a`.`id`'),
				),
			),
			'alias' => 'a',
			'cond' => array("`b`.`id` = '" . $id_theme . "'"),
			'fields' => array('`a`.*'),
		));
		if (count($forum) < 1)  return showInfoMessage(__('Can not find forum'), '/forum/' );

		// Check access to this forum. May be locked by pass or posts count
		$this->__checkForumAccess($forum[0]);
		
		
		
		if ($this->cached && $this->Cache->check($this->cacheKey)) {
			$html = $this->Cache->read($this->cacheKey);
		} else {
		
			// Получаем из БД информацию о теме
			$res = $this->DB->select('themes', DB_FIRST, array('cond' => array('id' => $id_theme)));
			// Если запрошенной темы не существует - возвращаемся на форум
			if (count($res) == 0) return showInfoMessage(__('Topic not found'), '/forum/' );
			
			$res = $this->Parser->quoteTags($res[0]);
			$theme_fetch = $res;
			$theme    = $res['title'];
			$locked   = $res['locked'];
			$id_forum = $res['id_forum'];
			$description = $res['description']; 

			// Заголовок страницы (содержимое тега title)
			$this->page_title = $this->page_title . ' - ' . h($theme);
			$html = '';
			$markets = array();
			


			$markets['{NAVIGATION}'] = get_link(__('Forums list'), '/forum/') . __('Separator') . get_link(h($forum[0]['title']), 
			'/forum/view_forum/' .  $id_forum) . __('Separator') . get_link(h($theme), '/forum/view_theme/' . $id_theme);
			if (!empty($description)) {
				$markets['{NAVIGATION }'] .= ' (' . h($description) . ')';
			}
			
			
			// Page nav
			$total = $this->DB->select('posts', DB_COUNT, array('cond' => array('id_theme' => $id_theme)));
			if ($total == 0) {
				$this->__delete_theme($id_theme);
				return showInfoMessage(__('Topic not found'), '/forum/view_forum/' . $id_forum );
			}

			// page nav
			$cntPages = ceil( $total / Config::read('posts_per_page', 'forum') );
			if (isset($_GET['page'])) {
				$page = (int)$_GET['page'];
				if ($page < 1) $page = 1;
			} else {
				$page = 1;
			}
			if ($page > $cntPages) $page = $cntPages;
			if ($cntPages > 1) {
				$markets['{PAGINATION}'] = pagination( $page, $cntPages, '/forum/view_theme/' . $id_theme );
				$this->page_title .= ' (' . $page . ')';
			} else {
				$markets['{PAGINATION}'] = '&nbsp;';
			}

			// Получаем из БД список сообщений (постов) темы
			$res = $this->DB->select('posts', DB_ALL, array(
				'joins' => array(
					array(
						'table' => 'users',
						'type' => 'LEFT',
						'alias' => 'b',
						'cond' => '`a`.`id_author` = `b`.`id`',
					),
					array(
						'table' => 'users',
						'type' => 'LEFT',
						'alias' => 'c',
						'cond' => '`a`.`id_editor` = `c`.`id`',
					),
				),
				'alias' => 'a',
				'order' => '`a`.`time` ASC, `a`.`id` ASC',
				'cond' => array('`a`.`id_theme`' => $id_theme),
				'page' => $page,
				'limit' => Config::read('posts_per_page', 'forum'),
				'fields' => array('a.id', 'a.message', 'a.id_author', 'a.time', 'a.attaches'
					, 'a.locked', 'a.id_theme',
					"DATE_FORMAT(a.edittime, '%d.%m.%Y') AS edittime", 'a.id_editor', "IFNULL(b.name, '" 
					. Config::read('not_reg_user', 'forum') . "') AS author", 'b.posts', 'b.url', 'b.rating',
					"DATE_FORMAT(b.puttime, '%d.%m.%Y') AS regtime", 'b.status AS status', 'b.state AS state',
					"IFNULL(b.signature, '') AS signature", "IFNULL(b.locked, 0) AS blocked"
					, "IFNULL(c.name, '') AS editor",
					"IFNULL(c.status, '') AS editor_status"),
			));
			if (count($res) == 0) {
				// Не может быть темы, в которой нет сообщений (постов) - надо ее удалить
				$this->__delete_theme($id_theme);
				return showInfoMessage(__('Topic not found'), '/forum/view_forum/' . $id_forum );
			}
			
			
			// ATTACHES
			$posts_ids = '';
			foreach ($res as $record) {
				$posts_ids .= $record['id'] . ' ,';
			}
			$posts_ids = substr($posts_ids, 0, (strlen($posts_ids) - 2));
			$posts_attaches = $this->DB->select('forum_attaches', DB_ALL, array(
				'cond' => array(
					'`post_id` IN (' . $posts_ids . ')',
				),
			));

			
			
			// Ссылка "Ответить" (если тема закрыта - выводим сообщение "Тема закрыта")
			if ( $locked == 0 ) {
				$markets['{ADD_LINK}'] = get_link(get_img('/sys/img/reply.png', array('alt' => __('Answer'), 
				'title' => __('Answer'))), '/forum/view_theme/' . $id_theme . '#sendForm');
			} else {
				$markets['{ADD_LINK}'] = get_img('/sys/img/reply_locked.png', 
				array('alt' => __('Theme locked'), 'title' => __('Theme locked')));
			}
			if (!isset($_SESSION['user'])) $markets['{ADD_LINK}'] = '';
			$markets['{META}'] = '';
			

			$info = null;
			$post_num = (($page - 1) * Config::read('posts_per_page', 'forum'));
			
			//serialize rating settings
			$rating_settings = $this->DB->select('users_settings', DB_FIRST, array(
				'cond' => array(
					'type' => 'rating',
				),
			));
			$rating_settings = (count($rating_settings) > 0) ? $rating_settings[0]['values'] : ''; 
			
			// render posts
			foreach ($res as $post) {
				$p_markets = array();
				$htmlpost = $this->Parser->getAndParse('vievpost.html');
				
				// Если автор сообщения (поста) - зарегистрированный пользователь
				if ($post['id_author']) {
					// Аватар
					if (is_file(R . 'sys/avatars/' . $post['id_author'] . '.jpg')) {
						$p_markets['{AVATAR}'] = get_url('/sys/avatars/' . $post['id_author'] . '.jpg');
					} else {
						$p_markets['{AVATAR}'] = get_url('/sys/img/null.gif');
					}
					  // Статус пользователя
					$status = $this->ACL->get_group_info();
					$user_status = $status[$post['status']];
					$p_markets['{USER_STATUS}'] = $user_status['title'];

					// Рейтинг пользователя (по количеству сообщений)
					$rating = $post['posts'];
					$rank_star = getUserRating($rating, $rating_settings);
					$p_markets['{RANK}'] = $rank_star['rank'];
					if (!empty($post['state'])) $p_markets['{RANK}'] = $post['state'];
					$p_markets['{USER_RANK}'] = get_img('/sys/img/' . $rank_star['img']);
					 
					
					
					// Если автор сообщения сейчас "на сайте"
					$users_on_line = getOnlineUsers(); 
					if (isset($users_on_line) &&  isset($users_on_line[$post['id_author']])) {
						$p_markets['{STATUS_ON_OFF}'] = get_img('/sys/img/online.png', array('title' => __('Online'), 'alt' => __('Online')));
					} else {
						$p_markets['{STATUS_ON_OFF}'] = get_img('/sys/img/offline.png', array('title' => __('Offline'), 'alt' => __('Offline')));
					}

					// Если пользователь заблокирован
					if ($post['blocked'])
						$p_markets['{STATUS_ON_OFF}'] = '<span class="statusBlock">' . __('Banned') . '</span>';

				} else { // Если автор сообщения - незарегистрированный пользователь
				    $p_markets['{AVATAR}'] = get_url('/sys/img/null.gif');
				}
				

				/****** ATTACH *****/
				$p_markets['{ATTACHMENT}'] = '';
				if(!empty($post['attaches'])) {
					if (!empty($posts_attaches)) {
						foreach ($posts_attaches as $singl_attach) {
							if ($singl_attach['post_id'] != $post['id']) continue;
							$collizion = false;
							
							
							if (file_exists(R . 'sys/files/forum/' . $singl_attach['filename'])) {
								$p_markets['{ATTACHMENT}'] .= __('Attachment') . $singl_attach['attach_number'] 
									. ': ' . get_img('/sys/img/file.gif', array('alt' => __('Open file'), 'title' => __('Open file'))) 
									. '&nbsp;' . get_link(($singl_attach['size'] / 1000) .' Кб', '/forum/download_file/' 
									. $singl_attach['filename'], array('target' => '_blank')) . '<br />';
									
									
								//if attach is image and isset markets for this image
								if ($singl_attach['is_image'] == 1) {
									$post['message'] = str_replace('{IMAGE' . $singl_attach['attach_number'] . '}', 
										'[img]' . get_url('/sys/files/forum/' . $singl_attach['filename']) . '[/img]',
										$post['message']);
								}
								$collizion = true;
								continue;
							}
						}
						/* may be collizion (paranoya mode) */
						if (!$collizion) $this->deleteCollizions($post);
					} else {
						$this->deleteCollizions($post);
					}
				}
				/******  END ATTACH  ******/
				
				
				//quote markets
				$post = $this->Parser->quoteTags($post);
				
				
				$p_markets['{SIGNATURE}'] = (!empty($post['signature'])) 
				? $this->Textarier->getSignature($post['signature'], $post['status']) : '' ;
				
				
				
				// Если автор сообщения (поста) - зарегистрированный пользователь
				$p_markets['{EMAIL}'] = '';
				$p_markets['{PRIVAT_MESSAGE}'] = '';
				if ( $post['id_author'] ) {
					$p_markets['{USER_PROFILE}'] = '&nbsp;' . get_link(get_img('/sys/img/icon_profile.gif', array('alt' => __('View profile'), 
						'title' => __('View profile'))), '/users/info/' . $post['id_author']);
					if ( isset( $_SESSION['user'] ) ) {
						$p_markets['{EMAIL}'] = '&nbsp;' . get_link(get_img('/sys/img/icon_email.gif', 
							array('alt' => __('Send mail'), 'title' => __('Send mail'))), 
							'/users/send_mail_form/' . $post['id_author']);
						$p_markets['{PRIVAT_MESSAGE}'] = '&nbsp;' . get_link(get_img('/sys/img/icon_pm.gif', 
							array('alt' => __('PM'), 'title' => __('PM'))), 
							'/users/send_msg_form/' . $post['id_author']);
					}
					$p_markets['{AUTHOR_SITE}'] = (!empty($post['url'])) ? '&nbsp;' . get_link(get_img('/sys/img/icon_www.gif', 
						array('alt' => __('Author site'), 'title' => __('Author site'))), h($post['url']), array('target' => '_blank'), true) : '';
				} else {
					$p_markets['{USER_PROFILE}'] = '<span class="details">' . get_img('/sys/img/null.gif') 
						. __('Unregister user') . '</span>';
				}
				
				
				// Если сообщение редактировалось...
				if (!empty($post['editor'])) {
					if ($post['id_author'] == $post['id_editor']) {
						$p_markets['{EDITOR}'] = __('Edit by author') . $post['edittime'];
					} else {
						$status_info = $this->ACL->get_user_group($post['editor_status']);
						$p_markets['{EDITOR}'] = __('Edited') . $post['editor'] . '(' 
							. $status_info['title'] . ') ' . $post['edittime'];
					}
				} else {
					$p_markets['{EDITOR}'] = '';
				}
				
				
				// User Rating
				$p_markets['{RATING}'] = $post['rating'];
				
				
				
				//edit and delete links
				$p_markets['{EDIT_LINK}'] = '';
				$p_markets['{DELETE_LINK}'] = '';
				if (isset($_SESSION['user'])) {
					if ($this->ACL->turn(array('forum', 'edit_posts'), false) 
					|| (!empty($_SESSION['user']['id']) && $post['id_author'] == $_SESSION['user']['id'] 
					&& $this->ACL->turn(array('forum', 'edit_mine_posts'), false))) {
						$p_markets['{EDIT_LINK}'] = '&nbsp;' . get_link(get_img('/sys/img/edit_16x16.png', 
						array('alt' => __('Edit'), 'title' => __('Edit'))), '/forum/edit_post_form/' . $post['id']);
					} 
					if ($this->ACL->turn(array('forum', 'delete_posts'), false) 
					|| (!empty($_SESSION['user']['id']) && $post['id_author'] == $_SESSION['user']['id'] 
					&& $this->ACL->turn(array('forum', 'delete_mine_posts'), false))) {
						$p_markets['{DELETE_LINK}'] = '&nbsp;' . get_link(get_img('/sys/img/delete_16x16.png', array('alt' => __('Delete'), 
						'title' => __('Delete'))), '/forum/delete_post/' . $post['id'], array('onClick' => "return confirm('" . __('Are you sure') . "')"));
					}
				}
				$p_markets['{ON_TOP}'] = '&nbsp;' . get_link(get_img('/sys/img/up_arrow_16x16.png', 
				array('alt' => __('To top'), 'title' => __('To top'))), '#top', array(), true);

				
				
				//message number
				$post_num++; $p_markets['{POST_NUMBER}'] = $post_num;
				$p_markets['{POST_NUMBER_URL}'] = 'http://' . $_SERVER['HTTP_HOST'] 
				. get_url('/forum/view_theme/' . $id_theme . '?page=' . $page . '#post' . $post_num, true);
				
				
				$message = $this->Parser->unquoteTags($post['message']);
				$message  = $this->Textarier->print_page($message, $post['status']);
				$p_markets['{MESSAGE}'] = $this->Parser->quoteTags($message);
				
				
		
				foreach($post as $key => $value) {
					if (!isset($p_markets['{' . strtoupper($key) . '}'])) 
						$p_markets['{' . strtoupper($key) . '}'] = h($value);
				}
			
				$info .= $this->_replaceMarkets($p_markets, $htmlpost);
				

				//set tags for cache
				$this->setCacheTag(array(
					'post_id_' . $post['id'],
					'user_id_' . $post['id_author'],
				));
			}
			$this->setCacheTag('theme_id_' . $id_theme);
			
			
			// Globalize navigation markets and clean variable
			$this->_globalize($markets);
			$markets = array();
			
			
			$htmltable = $this->Parser->getAndParse('themetable.html');
			$markets['{TITLE}'] = $theme;
			$markets['{POSTS}'] = $info;
			// Если тема не закрыта - выводим форму для быстрого ответа
			$markets['{REPLY_FORM}'] = $this->add_post_form($theme_fetch);
			
			
			$html = $this->_replaceMarkets($markets, $htmltable);
			
			//write into cache
			if ($this->cached)
				$this->Cache->write($html, $this->cacheKey, $this->cacheTags);
		}

		
		// Если страницу темы запросил зарегистрированный пользователь, значит он ее просмотрит
		if ( isset( $_SESSION['user'] ) and isset( $_SESSION['newThemes'] ) ) {
			if ( count( $_SESSION['newThemes'] ) > 0 ) {
				if ( in_array( $id_theme, $_SESSION['newThemes'] ) ) {
					unset( $_SESSION['newThemes'][$id_theme] );
				}
			} else {
				unset( $_SESSION['newThemes'] );
			}
		} 
		$this->DB->save('themes', array(
			'id' => $id_theme,
			'`views` = `views` + 1',
		));		
		//clean cache
		$this->Cache->clean(CACHE_MATCHING_TAG, array('action_viev_forum', 'theme_id_' . $id_theme));
		return $this->_view($html);
	}


	
	/**
	 * View last posts and last themes
	 * Build list with themes ordered by add date
	 *
	 * @return string html content
	 */
	public function last_posts() {
	
		$this->page_title .= ' - ' . __('Last update');
		$html = '';
    
		if ($this->cached && $this->Cache->check($this->cacheKey)) {
			$html = $this->Cache->read($this->cacheKey);
			return $this->_view($html);
        }
		
		
		// Page nav
		$nav = array();
		$total = $this->DB->select('themes', DB_COUNT);
		$perPage = Config::read('themes_per_page', 'forum');
		
		$cntPages = ceil( $total / $perPage );
		$page = (isset($_GET['page'])) ? (int)$_GET['page'] : 1;
		if ($page < 1) $page = 1;
		if ($page > $cntPages) $page = $cntPages;
		if ($cntPages > 1) {
			$nav['{PAGINATION}'] = pagination( $page, $cntPages, '/forum/last_posts/');
			$this->page_title .= ' (' . $page . ')';
		} else {
			$nav['{PAGINATION}'] = '&nbsp;';
		}
		
		$recOnPage = ($page == $cntPages) ? ($total % $perPage) : $perPage;
		$nav['{NAVIGATION}'] = get_link(__('Forums list'), '/forum/') . __('Separator') . __('Last update');
		$nav['{META}'] = __('Count all topics') . $total . '. ' . __('Count visible') . $recOnPage;
		$this->_globalize($nav);
		
		if ($total < 1) return $this->_view(__('No topics'));
		
		
		
		//get records
		$res = $this->DB->select('themes', DB_ALL, array(
			'joins' => array(
				array(
					'table' => 'users',
					'alias' => 'b',
					'cond' => '`b`.`id` = `a`.`id_author`',
					'type' => 'LEFT',
				),
				array(
					'table' => 'users',
					'alias' => 'c',
					'cond' => '`c`.`id` = `a`.`id_last_author`',
					'type' => 'LEFT',
				),
				array(
					'table' => 'forums',
					'alias' => 'd',
					'cond' => '`d`.`id` = `a`.`id_forum`',
					'type' => 'LEFT',
				),
			),
			'fields' => array('a.*', 'b.`name` AS author', 'c.`name` AS last_author', 'd.`title` AS forum_title'),
			'alias' => 'a',
			'order' => 'last_post DESC',
			'page' => $page,
			'limit' => Config::read('themes_per_page', 'forum'),
		));

		foreach ($res as $theme) {
			$markets = array(
				'{PARENT_FORUM}' => get_link($theme['forum_title']
				, '/forum/view_forum/' . $theme['id_forum']),
			);
			$template = $this->Parser->getAndParse('vievlastthemes.html');
			$row = $this->__parseThemeTable($theme, $template);
			$row = $this->_replaceMarkets($markets, $row);
			$html .= $row;

			
			
			//set cache tags
			$this->setCacheTag(array(
				'theme_id_' . $theme['id'],
			));
		}			
		
		
		$main_templ = $this->Parser->getAndParse('lastthemestable.html');
		$markets = array();
		$markets['{FORUM_NAME}'] = __('Last update');
		$markets['{THEMES}'] = $html;
		$html = $this->_replaceMarkets($markets, $main_templ);
		
		
		
		// write into cache
		if ($this->cached) {
			$this->Cache->write($html, $this->cacheKey, $this->cacheTags);
		}
		
		$this->_view($html);
	}
	
	



	/**
	 * Create HTML form for edit forum and paste current values into inputs
	 */
	public function edit_forum_form($id_forum = null) {
		//check access
		$this->ACL->turn(array('forum', 'edit_forums'));
		if (!isset($_SESSION['user'])) redirect('/forum/');
		$id_forum = intval($id_forum);
		if (empty($id_forum) || $id_forum < 1) redirect('/forum/');


		$html = '';
		$markets = array();
		

		// Получаем из БД информацию о форуме
		$query  = $this->DB->select('forums', DB_FIRST, array('cond' => array('id' => $id_forum)));
		$forum  = $query[0];
		$action = get_url('/forum/update_forum/' . $id_forum);

		
		// Если при заполнении формы были допущены ошибки
		if (isset($_SESSION['editForumForm'])) {
			$info        = $this->Parser->getAndParse('infomessage.html' );
			$info        = str_replace('{INFO_MESSAGE}', $_SESSION['editForumForm']['error'], $info );
			$html        = $html . $info . "\n";
			$title       = h($_SESSION['editForumForm']['title']);
			$description = h($_SESSION['editForumForm']['description']);			
			unset($_SESSION['editForumForm']);
		} else {
			$title       = h($forum['title']);
			$description = h($forum['description']);
		}
		
		
		// Считываем в переменную содержимое файла,
		// содержащего форму для редактирования форума
		$tpl = $this->Parser->getAndParse('editforumform.html');
		$markets['{ACTION}'] = $action;
		$markets['{TITLE}'] = $title;
		$markets['{DESCRIPTION}'] = $description;
		
		$markets = $this->Parser->quoteTags($markets);
		$tpl = $this->_replaceMarkets($markets, $tpl);
		$html = $html . $tpl;
		
		
		// nav block
		$navi = array();
		$navi['{NAVIGATION}'] = get_link(__('Forums list'), '/forum/') . __('Separator') . __('Edit forum');
		$this->_globalize($navi);
		

		return $this->_view($html);
	}





	/**
	 * Get request and work for it. Validate data and update record
	 */
	public function update_forum($id_forum = null) {
		//check access
		$this->ACL->turn(array('forum', 'edit_forums'));
		if (!isset($_SESSION['user']))  redirect('/forum/');
		$id_forum = intval($id_forum);
		if (empty($id_forum) || $id_forum < 1) redirect('/forum/');

		

		// Обрезаем переменные до длины, указанной в параметре maxlength тега input
		$title       = mb_substr($_POST['title'], 0, 120 );
		$description = mb_substr($_POST['description'], 0, 250 );
		$title       = trim($title);
		$description = trim($description);

		// Check fields fo empty values and valid chars
		$error = '';
		$valobj = new Validate;
		if (empty($title))                          
			$error = $error .'<li>'. __('Empty field "forum name"') .'</li>'. "\n";
		elseif (!$valobj->cha_val($title, V_TITLE)) 
			$error = $error .'<li>'. __('Wrong chars in "forum name"') .'</li>'. "\n";


		// if an errors
		if ( !empty( $error ) ) {
			$_SESSION['editForumForm']                = array();
			$_SESSION['editForumForm']['error']       = '<p class="errorMsg">' . __('Some error in form') . '</p>'.
														"\n" . '<ul class="errorMsg">' . "\n" . $error . '</ul>' . "\n";
			$_SESSION['editForumForm']['title']       = $title;
			$_SESSION['editForumForm']['description'] = $description;
			
			redirect('/forum/edit_forum_form/' . $id_forum);
		}
		
		$this->DB->save('forums', array(
			'title' => $title,
			'description' => $description,
			'id' => $id_forum
		));
		//clean cache
		$this->Cache->clean(CACHE_MATCHING_TAG, array('forum_id_' . $id_forum));
		$this->DB->cleanSqlCache();
		if ($this->Log) $this->Log->write('editing forum', 'forum id(' . $id_forum . ')');
		return showInfoMessage(__('Forum update is successful'), '/forum/' );
	}





	/**
	* raise forum
	*
	* @id_forum (int)    forum ID
	* @return           info message
	*/
	public function forum_up($id_forum = null) {
		//check access
		$this->ACL->turn(array('forum', 'replace_forums'));
		if (!isset($_SESSION['user']))  {
			redirect('/forum/');
		}
		$id_forum = intval($id_forum);
		if (empty($id_forum) || $id_forum < 1) {
			redirect('/forum/');
		}

		
		// upper forum
		$id_forum_up = intval($id_forum);
		$res = $this->DB->select('forums', DB_FIRST, array('cond' => array('id' => $id_forum_up)));
		// upper position
		$order_up = $res[0]['pos'];
		$res = $this->DB->select('forums', DB_FIRST, array(
			'cond' => array(
				'pos < ' . $order_up, 
				'in_cat' => $res[0]['in_cat']
			),
			'order' => 'pos DESC',
			'fields' => array('id', 'pos'),
		));
		if ( count($res) == 0 ) return showInfoMessage(__('Forum is above all'), '/forum/' );
		
		// Порядок следования и ID форума, который находится выше и будет "опущен" вниз
		// ( поменявшись местами с форумом, который "поднимается" вверх )
		$id_forum_down = $res[0]['id'];
		$order_down    = $res[0]['pos'];
		
		// replace forums
		$res1 = $this->DB->save('forums', array(
				'pos' => $order_down,
				'id'  => $id_forum_up
		));
		$res2 = $this->DB->save('forums', array(
				'pos' => $order_up,
				'id'  => $id_forum_down
		));
		
		//clean cache
		$this->Cache->clean(CACHE_MATCHING_TAG, array('forum_id_' . $id_forum));
		$this->DB->cleanSqlCache();
		
		if ($this->Log) $this->Log->write('uping forum', 'forum id(' . $id_forum . ')');
		if ( $res1 and $res2 )
			return showInfoMessage(__('Operation is successful'), '/forum/' );
		else
			return showInfoMessage(__('Some error occurred'), '/forum/' );
	}




	/**
	* down forum
	*
	* @id_forum (int)    forum ID
	* @return           info message
	*/
	public function forum_down($id_forum = null) {
		//check access
		$this->ACL->turn(array('forum', 'replace_forums'));
		if (!isset($_SESSION['user']))  {
			redirect('/forum/');
		}
		$id_forum = intval($id_forum);
		if (!isset($id_forum)) {
			redirect('/forum/');
		}
	
		
		// downing forum
		$id_forum_down = $id_forum;
		$res = $this->DB->select('forums', DB_FIRST, array('cond' => array('id' => $id_forum_down)));
		
		// Порядок следования форума, который "опускается" вниз
		$order_down = $res[0]['pos'];
		$res = $this->DB->select('forums', DB_FIRST, array(
			'cond' => array(
				'in_cat' => $res[0]['in_cat'],
				'pos > ' . $order_down,
			),
			'order' => 'pos',
			'fields' => array('id', 'pos'),
		));
		if (count($res) == 0) return true;
		
		// Порядок следования и ID форума, который находится ниже и будет "поднят" вверх
		// ( поменявшись местами с форумом, который "опускается" вниз )
		$id_forum_up = $res[0]['id'];
		$order_up    = $res[0]['pos'];
		// replace forums
		$res1 = $this->DB->save('forums', array(
				'pos' => $order_down,
				'id'  => $id_forum_up
		));
		$res2 = $this->DB->save('forums', array(
				'pos' => $order_up,
				'id'  => $id_forum_down
		));
		
		//clean cache
		$this->Cache->clean(CACHE_MATCHING_TAG, array('forum_id_' . $id_forum));
		$this->DB->cleanSqlCache();
		if ($this->Log) $this->Log->write('down forum', 'forum id(' . $id_forum . ')');
		if ( $res1 and $res2 )
			return showInfoMessage(__('Operation is successful'), '/forum/' );
		else
			return showInfoMessage(__('Some error occurred'), '/forum/' );
	}




	/**
	* delete forum
	*
	* @id_forum (int)    forum ID
	* @return            info message
	*/
	public function delete_forum($id_forum = null) {
		//check access
		$this->ACL->turn(array('forum', 'delete_forums'));
		if ( !isset( $_SESSION['user'] ) ) {
			redirect('/forum/');
		}
		$id_forum = (int)$id_forum;
		if (empty($id_forum) || $id_forum < 1) {
			redirect('/forum/');
		}
	
		
		// Можно удалить только форум, который не содержит тем (в целях безопасности)
		$res = $this->DB->select('themes', DB_COUNT, array('cond' => array('id_forum' => $id_forum)));
		if ($res > 0)
			return showInfoMessage(__('Can not delete forum with themes'), '/forum/' );
		else
			$this->DB->query("DELETE FROM `" . $this->DB->getFullTableName('forums') . "` WHERE `id`='{$id_forum}'");
		//clean cache
		$this->Cache->clean(CACHE_MATCHING_TAG, array('forum_id_' . $id_forum));
		$this->DB->cleanSqlCache();
		if ($this->Log) $this->Log->write('delete forum', 'forum id(' . $id_forum . ')');
		return showInfoMessage(__('Operation is successful'), '/forum/' );
	}





	/**
	* form per add theme into forum
	*
	* @id_forum (int)    forum ID
	* @return            html content
	*/
	public function add_theme_form($id_forum = null) {
		//check access
		$this->ACL->turn(array('forum', 'add_themes'));
		$id_forum = intval($id_forum);
		if (empty($id_forum) || $id_forum < 1) redirect('/forum/');
		$writer_status = (!empty($_SESSION['user']['status'])) ? $_SESSION['user']['status'] : 0;

		
		
		$targ_forum = $this->DB->select('forums', DB_FIRST, array('cond' => array('id' => $id_forum)));
		if (count($targ_forum) != 1) redirect('/forum/');
		$forum = $targ_forum[0];
		
		
		// Check access to this forum. May be locked by pass or posts count
		$this->__checkForumAccess($forum);
		
		
		$html    = '';
		$markets = array();
		
		// preview
		if (isset($_SESSION['viewMessage']) and !empty($_SESSION['viewMessage']['message'])) {
			$view    = $this->Parser->getAndParse('previewmessage.html');
			$view    = str_replace( '{MESSAGE}', 
				$this->Textarier->print_page($_SESSION['viewMessage']['message'], $writer_status), $view );
			$html    = $html . $view . "\n";
			$theme   = h($_SESSION['viewMessage']['theme']);
			$description = h($_SESSION['viewMessage']['description']);
			$message = $_SESSION['viewMessage']['message'];			
			unset( $_SESSION['viewMessage'] );
		}

		// errors
		if (isset($_SESSION['addThemeForm'])) {
			$info    = $this->Parser->getAndParse('infomessage.html');
			$info    = str_replace('{INFO_MESSAGE}', $_SESSION['addThemeForm']['error'], $info);
			$html    = $html . $info . "\n";
			$theme   = h($_SESSION['addThemeForm']['theme']);
			$description = h($_SESSION['addThemeForm']['description']); 
			$message = $_SESSION['addThemeForm']['message'];			
			unset($_SESSION['addThemeForm']);
		}

		
		
		$markets['{ACTION}'] = get_url('/forum/add_theme/' . $id_forum);
		$html = $html . $this->Parser->getAndParse('addthemeform.html' );
		$markets['{THEME}'] = (!empty($theme)) ? $theme : '';
		$markets['{DESCRIPTION}'] = (!empty($description)) ? $description : ''; 
		$markets['{MAIN_TEXT}'] = (!empty($message)) ? $message : '';
		$html = $this->_replaceMarkets($markets, $html);

			
		
		// nav block
		$navi = array();
		$navi['{NAVIGATION}'] = get_link(__('Forums list'), '/forum/') . __('Separator') 
			. get_link(h($forum['title']), '/forum/view_forum/' . $id_forum);
		$this->_globalize($navi);
		
		
		return $this->_view($html);
	}





	/**
	* add theme into forum
	*
	* @id_forum (int)    forum ID
	* @return            info message
	*/
	public function add_theme($id_forum = null) {
		//check access
		$this->ACL->turn(array('forum', 'add_themes'));
		if (!isset($id_forum) || !isset($_POST['theme']) || !isset($_POST['mainText'])) {
			redirect('/forum/');
		}
		$id_forum = intval($id_forum);
		if (empty($id_forum) || $id_forum < 1) {
			redirect('/forum/');
		}


		$targ_forum = $this->DB->select('forums', DB_FIRST, array('cond' => array('id' => $id_forum)));
		if (count($targ_forum) != 1) redirect('/forum/');
		$forum = $targ_forum[0];
		
		
		// Check access to this forum. May be locked by pass or posts count
		$this->__checkForumAccess($forum);
		
		
		// cut lenght
		$theme   = mb_substr($_POST['theme'], 0, 55);
		$message = $_POST['mainText'];
		$theme   = trim($theme);
		$description = trim(mb_substr($_POST['description'], 0, 128)); 
		$message = trim($message);

		// preview
		if (isset($_POST['viewMessage'])) {
			$_SESSION['viewMessage']['theme']   = $theme;
			$_SESSION['viewMessage']['description'] = $description; 
			$_SESSION['viewMessage']['message'] = $message;
			redirect('/forum/add_theme_form/' . $id_forum );
		}

		// Check fields of empty values and valid chars
		$error = '';
		$valobj = new Validate;
		if (empty($theme)) 
			$error = $error . '<li>' . __('Empty field "theme"') . '</li>'."\n";
		elseif (!$valobj->cha_val($theme, V_TITLE)) 
			$error = $error. '<li>' . __('Wrong chars in "theme"').'</li>'."\n";
		if (empty($message)) 
			$error = $error. '<li>' . __('Empty field "message"').'</li>'."\n";
		if (mb_strlen($message) > Config::read('max_post_lenght', 'forum')) 
			$error = $error . '<li>' . sprintf(__('Field "message" contains more symbols'), Config::read('max_post_lenght', 'forum')) . '</li>'."\n";

		for ($i = 1; $i < 6; $i++) {
			if (!empty($_FILES['attach' . $i]['name'])) {
				if ($_FILES['attach' . $i]['size'] > Config::read('max_file_size')) {
					$error = $error . '<li>' . sprintf(__('Wery big file'), $i, (Config::read('max_file_size')/1024)) . '</li>'."\n";
				}
			}
		}
		// errors
		if (!empty($error)) {
			$_SESSION['addThemeForm']            = array();
			$_SESSION['addThemeForm']['error']   = '<p class="errorMsg">' . __('Some error in form') . '</p>'.
												 "\n" . '<ul class="errorMsg">' . "\n" . $error . '</ul>' . "\n";
			$_SESSION['addThemeForm']['theme']   = $theme;
			$_SESSION['addThemeForm']['description'] = $description; 
			$_SESSION['addThemeForm']['message'] = $message;
			redirect('/forum/add_theme_form/' . $id_forum );
		}
		
		$message = mb_substr($message, 0, Config::read('max_post_lenght', 'forum'));
		$res = $this->DB->save('themes', array(
				'title'          => $theme,
				'description'    => $description,
				'id_author'      => $_SESSION['user']['id'],
				'time'           => new Expr('NOW()'),
				'id_last_author' => $_SESSION['user']['id'],
				'last_post'      => new Expr('NOW()'),
				'id_forum'       => $id_forum 
		));
		$id_theme = mysql_insert_id();

		// add first post
		$res = $this->DB->save('posts', array(
				'message'        => $message,
				'id_author'      => $_SESSION['user']['id'],
				'time'           => new Expr('NOW()'),
				'edittime'       => new Expr('NOW()'),
				'id_theme'       => $id_theme 
		));
		$post_id = mysql_insert_id();
		
		
		/***** END ATTACH *****/
		$attaches_exists = 0;
		// Массив недопустимых расширений файла вложения
		$extentions = array('.php', '.phtml', '.php3', '.html', '.htm', '.pl', '.PHP', '.PHTML', '.PHP3', '.HTML', '.HTM', '.PL');
		$img_extentions = array('.png','.jpg','.gif','.jpeg', '.PNG','.JPG','.GIF','.JPEG');
		/* delete collizions if exists */
		$this->deleteCollizions(array('id' => $post_id), true);
		for ($i = 1; $i < 6; $i++) {
			$attach_name = 'attach' . $i;
			if (!empty($_FILES[$attach_name]['name'])) {
				// Извлекаем из имени файла расширение
				$ext = strrchr($_FILES[$attach_name]['name'], ".");
				// Формируем путь к файлу
				if (in_array( $ext, $extentions))
					$file = $post_id . '-' . $i . '-' . date("YmdHi") . '.txt';
				else
					$file = $post_id . '-' . $i . '-' . date("YmdHi") . $ext;
				$is_image = (in_array($ext, $img_extentions)) ? 1 : 0;
				// Перемещаем файл из временной директории сервера в директорию files
				if (move_uploaded_file($_FILES[$attach_name]['tmp_name'], R . 'sys/files/forum/' . $file)) {
					chmod(R . 'sys/files/forum/' . $file, 0644);
					$attach_file_data = array(
						'post_id'       => $post_id,
						'theme_id'      => $id_theme,
						'user_id'       => $_SESSION['user']['id'],
						'attach_number' => $i,
						'filename'      => $file,
						'size'          => $_FILES[$attach_name]['size'],
						'date'          => new Expr('NOW()'),
						'is_image'      => $is_image,
					);
					if ($this->DB->save('forum_attaches', $attach_file_data)) {
						$attaches_exists = 1;
					}
				}
			}
		}
		if ($attaches_exists == 1) {
			$this->DB->save('posts', array(
				'id'       => $post_id,
				'attaches' => 1
			));
		}
		/***** END ATTACH *****/
		
		
		// Обновляем число оставленных сообщений и созданных тем
		$this->DB->save('users', array(
			'`themes` = `themes` + 1',
			'`posts` = `posts` + 1',
			'id' => $_SESSION['user']['id'],
		));
		$this->DB->save('forums', array(
			'`themes` = `themes` + 1',
			'`posts` = `posts` + 1',
			'last_theme_id' => $id_theme,
			'id' => $id_forum,
		));
		//clean cache
		$this->Cache->clean(CACHE_MATCHING_ANY_TAG, array(
			'user_id_' . $_SESSION['user']['id'],
			'forum_id_' . $id_forum,
		));
		$this->DB->cleanSqlCache();
		if ($this->Log) $this->Log->write('adding theme', 'theme id(' . $id_theme . '), post id(' . $post_id . ')');
		return showInfoMessage(__('Operation is successful'), '/forum/view_forum/' . $id_forum );
	}






	/**
	* form per edit theme
	*
	* @id_forum (int)    theme ID
	* @return            html content
	*/
	public function edit_theme_form($id_theme = null) {
		$id_theme = (int)$id_theme;
		if (empty($id_theme) || $id_theme < 1) redirect('/forum/');


		// Получаем из БД информацию о редактируемой теме
		$res = $this->DB->select('themes', DB_FIRST, array(
			'joins' => array(
				array(
					'table' => 'users',
					'alias' => 'b',
					'type' => 'LEFT',
					'cond' => '`b`.`id` = `a`.`id_author`',
				),
			),
			'cond' => array('`a`.`id`' => $id_theme),
			'alias' => 'a',
			'fields' => array('a.*', 'b.`name` as author'),
		));
		if (count($res) == 0) redirect('/forum/');
		
		$theme = $res[0];
		$id_forum = $theme['id_forum'];
		$html = '';
		$markets = array();
		
		
		//check access
		if (!$this->ACL->turn(array('forum', 'edit_themes'), false) 
		&& (empty($_SESSION['user']['id']) || $theme['id_author'] != $_SESSION['user']['id'] 
		|| !$this->ACL->turn(array('forum', 'edit_mine_themes'), false))) {
			return showInfoMessage(__('Permission denied'), '/forum/view_forum/' . $id_forum );
		}

		
		// Если при заполнении формы были допущены ошибки
		if (isset($_SESSION['editThemeForm'])) {
			$info    = $this->Parser->getAndParse('infomessage.html');
			$info    = str_replace('{INFO_MESSAGE}', $_SESSION['editThemeForm']['error'], $info);
			$html    = $info . $html . "\n";
			$name = h($_SESSION['editThemeForm']['name']);
			$description = h($_SESSION['editThemeForm']['description']); 
			unset($_SESSION['editThemeForm']);
		} else {
			$name = h($theme['title']);
			$description = h($theme['description']); 
		}

		// Формируем список форумов, чтобы можно было переместить тему в другой форум
		$res = $this->DB->select('forums', DB_ALL, array('order' => 'pos'));
		if (count($res) == 0) redirect('/forum/');


		$options = '';
		foreach ($res as $forum) {
			if ($forum['id'] == $theme['id_forum'])
				$options = $options.'<option value="'.$forum['id'].'" selected>'.h($forum['title']).'</option>'."\n";
			else
				$options = $options.'<option value="'.$forum['id'].'">'.h($forum['title']).'</option>'."\n";
		}


		// Считываем в переменную файл шаблона, содержащего форму для редактирования темы
		$tpl = $this->Parser->getAndParse('editthemeform.html' );
		$markets['{ACTION}'] = get_url('/forum/update_theme/' . $id_theme);
		$markets['{NAME}'] = $name;
		$markets['{DESCRIPTION}'] = $description; 
		$markets['{AUTHOR}'] = h($theme['author']);
		$markets['{OPTIONS}'] = $options;
		
		$markets = $this->Parser->quoteTags($markets);
		$html = $html . $this->_replaceMarkets($markets, $tpl);

		
		// nav block
		$navi = array();
		$navi['{NAVIGATION}'] = get_link(__('Forums list'), '/forum/') . __('Separator') . __('Edit theme');
		$this->_globalize($navi);
		
		
		return $this->_view($html);
	}





	/**
	* update theme
	*
	* @id_forum (int)    theme ID
	* @return            info message
	*/
	public function update_theme($id_theme = null) {
		
		// Если не переданы данные формы - функция вызвана по ошибке
		if (!isset($id_theme) || !isset($_POST['id_forum']) || !isset($_POST['name'])) {
			redirect('/forum/');
		}
		$id_theme = (int)$id_theme;
		$id_forum = (int)$_POST['id_forum'];
		if ( $id_theme < 1 || $id_forum < 1) {
			redirect('/forum/');
		}

		// Обрезаем переменные до длины, указанной в параметре maxlength тега input
		$name = mb_substr($_POST['name'], 0, 55);
		$name = trim($name);
		$description = trim(mb_substr($_POST['description'], 0, 128)); 
		
		
		// validate ...
		$error = '';
		$valobj = new Validate;
		if (empty($name))                  
			$error = $error. '<li>' . __('Empty field "theme"') . '</li>'."\n";
		if (!$valobj->cha_val($name, V_TITLE)) 
			$error = $error. '<li>' . __('Wrong chars in "theme"') . '</li>'."\n";

		// errors
		if (!empty($error)) {
			$_SESSION['editThemeForm']           = array();
			$_SESSION['editThemeForm']['error']  = '<p class="errorMsg">' . __('Some error in form') 
			. '</p>' . "\n" . '<ul class="errorMsg">'."\n".$error.'</ul>'."\n";
			$_SESSION['editThemeForm']['name']   = $name;
			$_SESSION['editThemeForm']['description'] = $description; 
			redirect('/forum/edit_theme_form/' . $id_theme);
		}
		
		// if theme moved into another forum 
		$res = $this->DB->select('themes', DB_FIRST, array('cond' => array('id' => $id_theme)));
		if (count($res) < 1) return showInfoMessage(__('Theme does not exists'), '/forum/');
		$from_forum = $res[0]['id_forum'];
		
		//check access
		if (!$this->ACL->turn(array('forum', 'edit_themes'), false) 
		&& (empty($_SESSION['user']['id']) || $theme['id_author'] != $_SESSION['user']['id'] 
		|| !$this->ACL->turn(array('forum', 'edit_mine_themes'), false))) {
			return showInfoMessage(__('Permission denied'), '/forum/view_forum/' . $id_forum );
		}
		
		
		// update theme
		$this->DB->save('themes', array(
			'title' => $name,
			'description' => $description,
			'id_forum' => $id_forum,
			'id' => $id_theme,
		));
		
		//update forums info
		if ($from_forum != $id_forum) {
			$new_forum = $this->DB->select('forums', DB_COUNT, array('cond' => array('id' => $id_forum)));
			if ($new_forum < 1) return showInfoMessage(__('No forum for moving'), '/forum/');
			
			
			$posts_cnt = $this->DB->select('posts', DB_COUNT, array('cond' => array('id_theme' => $id_theme)));
			$this->DB->save('forums', array(
				'`posts` = `posts` - ' . $posts_cnt,
				'`themes` = `themes` - 1',
				'id' => $from_forum,
			));
			$this->DB->save('forums', array(
				'`posts` = `posts` + ' . $posts_cnt,
				'`themes` = `themes` + 1',
				'id' => $id_forum,
			));		
			$this->DB->query("UPDATE `" . $this->DB->getFullTableName('forums') . "` as forum SET 
				forum.`last_theme_id` = IFNULL((SELECT `id` FROM `" . $this->DB->getFullTableName('themes') . "` 
				WHERE `id_forum` = forum.`id` ORDER BY `last_post` DESC LIMIT 1), 0) 
				WHERE forum.`id` IN ('" . $from_forum . "', '" . $id_forum . "')");
		}

				
		//clean cahce
		$this->Cache->clean(CACHE_MATCHING_ANY_TAG, array('theme_id_' . $id_theme));
		$this->DB->cleanSqlCache();
		if ($this->Log) $this->Log->write('editing theme', 'theme id(' . $id_theme . ')');
		return showInfoMessage(__('Operation is successful'), '/forum/view_forum/' . $id_forum );
		
	}


	
	


	/**
	 * Deleting theme
	 */
	public function delete_theme($id_theme = null) {
		$id_theme = intval($id_theme);
		if (empty($id_theme) || $id_theme < 1) {
			redirect('/forum/');
		}

		
		// if theme moved into another forum 
		$res = $this->DB->select('themes', DB_FIRST, array('cond' => array('id' => $id_theme)));
		if (count($res) < 1) return showInfoMessage(__('Topic not found'), '/forum/');
		//check access
		if (!$this->ACL->turn(array('forum', 'delete_themes'), false) 
		|| (!empty($_SESSION['user']['id']) && $res[0]['id_author'] == $_SESSION['user']['id'] 
		&& !$this->ACL->turn(array('forum', 'delete_mine_themes'), false))) {
			return showInfoMessage(__('Permission denied'), '/forum/');
		}

		// delete colision ( this is paranoia )
		$this->DB->query("DELETE FROM `" . $this->DB->getFullTableName('themes') . "` WHERE id NOT IN (SELECT DISTINCT id_theme FROM `posts`)");
		$this->DB->query("DELETE FROM `" . $this->DB->getFullTableName('posts') . "` WHERE id_theme NOT IN (SELECT id FROM `themes`)");

		
		
		// Сперва мы должны удалить все сообщения (посты) темы;
		// начнем с того, что удалим файлы вложений
		$res = $this->DB->select('posts', DB_ALL, array('cond' => array('id_theme' => $id_theme)));
		if (count($res) > 0) {
			foreach ($res as $file) {
				// Удаляем файл, если он есть
				$attach_files = $this->DB->select('forum_attaches', DB_ALL, array('cond' => array('post_id' => $file['id'])));
				if (count($attach_files) > 0) {
					foreach ($attach_files as $attach_file) {
						if (file_exists(R . 'sys/files/forum/' . $attach_file['filename'])) {
							if (@unlink(R . 'sys/files/forum/' . $attach_file['filename'])) {
								$this->DB->query("DELETE FROM `" . $this->DB->getFullTableName('forum_attaches') 
								. "` WHERE `id`='" . $attach_file['id'] . "'");
							}
						}
					}
				}
				// заодно обновляем таблицу USERS
				if ( $file['id_author'] ) {
					$this->DB->save('users', array(
						'`posts` = `posts` - 1',
						'id' => $file['id_author'],
					));
				}
			}
		}
		
		
		$attach_files = $this->DB->select('forum_attaches', DB_ALL, array('cond' => array('theme_id' => $id_theme)));
		if (count($attach_files) > 0) {
			foreach ($attach_files as $attach_file) {
				if (file_exists(R . 'sys/files/forum/' . $attach_file['filename'])) {
					if (@unlink(R . 'sys/files/forum/' . $attach_file['filename'])) {
						$this->DB->query("DELETE FROM `" . $this->DB->getFullTableName('forum_attaches') . "` WHERE `id`='" . $attach_file['id'] . "'");
					}
				}
			}
		}
		
		
		
		//we must know id_forum
		$theme = $this->DB->select('themes', DB_FIRST, array('cond' => array('id' => $id_theme)));
		//delete info
		$p_res = $this->DB->query("DELETE FROM `" . $this->DB->getFullTableName('posts') . "` WHERE `id_theme` = '" . $id_theme . "'");
		$t_res = $this->DB->query("DELETE FROM `" . $this->DB->getFullTableName('themes') . "` WHERE `id` = '" . $id_theme . "'");
		
		//update info
		if (count($theme) > 0) {
			// Обновляем таблицу USERS
			$u_res = $this->DB->query(
				"UPDATE `" . $this->DB->getFullTableName('users') . "` SET 
				`themes` = (SELECT COUNT(*) FROM `" . $this->DB->getFullTableName('themes') . "` 
				WHERE `id_author` = '" . $theme[0]['id_author'] . "')
				, `posts` = (SELECT COUNT(*) FROM `" . $this->DB->getFullTableName('posts') . "` 
				WHERE `id_author` = '" . $theme[0]['id_author'] . "')
				WHERE `id` = '" . $theme[0]['id_author'] . "'");

			//update forum info
			$this->DB->query(
				"UPDATE `" . $this->DB->getFullTableName('forums') . "` SET `themes` = 
				(SELECT COUNT(*) FROM `" . $this->DB->getFullTableName('themes') . "` 
				WHERE `id_forum` = '" . $theme[0]['id_forum'] . "'), `posts` = 
				(SELECT COUNT(b.`id`) FROM `" . $this->DB->getFullTableName('themes') . "` a 
				LEFT JOIN `" . $this->DB->getFullTableName('posts') . "` b ON a.`id`=b.`id_theme`
				WHERE a.`id_forum` = '" . $theme[0]['id_forum'] . "'),
				`last_theme_id`=(SELECT `id` FROM `" . $this->DB->getFullTableName('themes') . "` 
				WHERE `id_forum`='" . $theme[0]['id_forum'] . "'
				ORDER BY `last_post` DESC  LIMIT 1) WHERE `id` = '" . $theme[0]['id_forum'] . "'" );
		}
		
		//clean cache
		$this->Cache->clean(CACHE_MATCHING_ANY_TAG, array('theme_id_' . $id_theme,));
		$this->Cache->clean(CACHE_MATCHING_TAG, array('module_forum', 'action_index'));
		$this->DB->cleanSqlCache();
		if ($this->Log) $this->Log->write('delete theme', 'theme id(' . $id_theme . ')');
		return showInfoMessage(__('Theme is deleted'), '/forum/view_forum/' . $theme[0]['id_forum'] );
	}




	/**
	 * Close Theme
	 */
	public function lock_theme($id_theme = null) {
		$this->ACL->turn(array('forum', 'close_themes'));
		$id_theme = (int)$id_theme;
		if ($id_theme < 1) {
			redirect('/forum/');
		}
		
		$theme = $this->DB->select('themes', DB_FIRST, array('cond' => array('id' => $id_theme)));
		if (count($theme) < 1) return showInfoMessage(__('Topic not found'), '/forum/' );
		
		// Сначала заблокируем сообщения (посты) темы
		$this->DB->save('posts', array(
			'locked' => '1',
		), array(
			'id_theme' => $id_theme,
		));
		// Теперь заблокируем тему
		$this->DB->save('themes', array(
			'id' => $id_theme,
			'locked' => 1
		));
		
		//clean cache
		$this->Cache->clean(CACHE_MATCHING_ANY_TAG, array('theme_id_' . $id_theme));
		$this->DB->cleanSqlCache();
		if ($this->Log) $this->Log->write('lock theme', 'theme id(' . $id_theme . ')');
		return showInfoMessage(__('Theme is locked'), '/forum/view_forum/' . $theme[0]['id_forum'] );
	}





	/**
	 * Unlocking Theme
	 */
	public function unlock_theme($id_theme = null) {
		$this->ACL->turn(array('forum', 'close_themes'));
		$id_theme = (int)$id_theme;
		if ($id_theme < 1) {
			redirect('/forum/');
		}

		$theme = $this->DB->select('themes', DB_FIRST, array('cond' => array('id' => $id_theme)));
		if (count($theme) < 1) return showInfoMessage(__('Topic not found'), '/forum/' );
		
		// Сначала разблокируем сообщения (посты) темы
		$this->DB->save('posts', array(
			'locked' => '0',
		), array(
			'id_theme' => $id_theme,
		));
		// Теперь разблокируем тему
		$this->DB->save('themes', array(
				'id' => $id_theme,
				'locked' => 0
		));
		
		//clean cache
		$this->Cache->clean(CACHE_MATCHING_ANY_TAG, array('theme_id_' . $id_theme));
		$this->DB->cleanSqlCache();
		if ($this->Log) $this->Log->write('unlock theme', 'theme id(' . $id_theme . ')');
		return showInfoMessage(__('Theme is open'), '/forum/view_forum/' . $theme[0]['id_forum'] );
	}





	/**
	 * Create reply form
	 *
	 * @param array $theme Theme info
	 * @return string HTML reply form
	 */ 
	private function add_post_form($theme = null) {
		if (empty($theme)) return null;
		$id_theme = (int)$theme['id'];
		if ($id_theme < 1) return null;
		$writer_status = (!empty($_SESSION['user']['status'])) ? $_SESSION['user']['status'] : 0;


		if ($this->ACL->turn(array('forum', 'add_posts'), false)) {
			if ($theme['locked'] == 1) {
				$html = '<div class="not-auth-mess">' . __('Theme is locked') . '</div>';
			} else {


				$message = '';
				$html = '';
				$markets = array();

				if (isset($_SESSION['viewMessage']) and !empty($_SESSION['viewMessage'])) {
					$view    = $this->Parser->getAndParse('previewmessage.html');
					$view    = str_replace( '{MESSAGE}', 
						$this->Textarier->print_page($_SESSION['viewMessage'], $writer_status), $view);
					$html    = $html . $view . "<script>window.location.href=\"#preview\";</script>\n";
					$message = h($_SESSION['viewMessage']);
					unset($_SESSION['viewMessage']);
				}

				// Если при заполнении формы были допущены ошибки
				if (isset($_SESSION['addPostForm'])) {
					$info    = $this->Parser->getAndParse('infomessage.html');
					$info    = str_replace('{INFO_MESSAGE}', $_SESSION['addPostForm']['error'], $info);
					$html    = $html . $info . "\n";
					$message = h($_SESSION['addPostForm']['message']);
					unset($_SESSION['addPostForm']);
				}

				$tpl    = $this->Parser->getAndParse('replyform.html');
				$tpl = $html . $tpl;
				$markets['{ACTION}'] = get_url('/forum/add_post/' . $id_theme);
				$markets['{MAIN_TEXT}'] = $message;
				$html = $this->_replaceMarkets($markets, $tpl);
			}
			
			
		} else {
			if (isset($_SESSION['user']))
				$html = '<div class="not-auth-mess">' . __('Dont have permission to write post') . '</div>';
			else
				$html = '<div class="not-auth-mess">' . __('Guests cant write posts') . '</div>';
		}
		
		return $html;
	}





	/**
	 * Adding new record into posts table
	 *
	 * @param int $id_theme
	 */
	public function add_post($id_theme = null) {
		$this->ACL->turn(array('forum', 'add_posts'));
		if (empty($id_theme) || !isset($_POST['mainText'])) { 
			redirect('/forum/');
		}
		$id_theme = (int)$id_theme;
		if ($id_theme < 1) {
			redirect('/forum/');
		}

		
		// Check access to this forum. May be locked by pass or posts count
		$forum = $this->DB->select('forums', DB_FIRST, array(
			'joins' => array(
				array(
					'table' => 'themes',
					'alias' => 'b',
					'type' => 'LEFT',
					'cond' => array('`b`.`id_forum` = `a`.`id`'),
				),
			),
			'alias' => 'a',
			'cond' => array("`b`.`id` = '" . $id_theme . "'"),
			'fields' => array('`a`.*'),
		));
		if (count($forum) < 1)  return showInfoMessage(__('Can not find forum'), '/forum/' );
		$this->__checkForumAccess($forum[0]);
		
		
		
		// Проверяем, не заблокирована ли тема?
		$res = $this->DB->select('themes', DB_FIRST, array('cond' => array('id' => $id_theme)));
		if (count($res) == 0) redirect('/forum/');
		if ($res[0]['locked'] == 1)
			return showInfoMessage(__('Can not write in a closed theme'), 
			'/forum/view_theme/' . $id_theme );


		// Обрезаем сообщение (пост) до длины $set['forum']['max_post_lenght']
		$message = trim($_POST['mainText']);
		// Если пользователь хочет посмотреть на сообщение перед отправкой
		if ( isset( $_POST['viewMessage'] ) ) {
			$_SESSION['viewMessage'] = $message;
			redirect('/forum/view_theme/' . $id_theme);
		}

		// Проверяем, правильно ли заполнены поля формы
		$error = '';
		if (empty($message)) $error = $error . '<li>' . __('Empty field "message"') . '</li>'."\n";
		if (strlen($message) > Config::read('max_post_lenght', 'forum')) 
			$error = $error . '<li>' . sprintf(__('Field "message" contains more symbols'), Config::read('max_post_lenght', 'forum')) . '</li>'."\n";
		
		
		$gluing = true;
		for ($i = 1; $i < 6; $i++) {
			if (!empty($_FILES['attach' . $i]['name'])) {
				if ($_FILES['attach' . $i]['size'] > Config::read('max_file_size')) {
					$error = $error . '<li>' . sprintf(__('Wery big file'), $i, (Config::read('max_file_size')/1024)) . '</li>'."\n";
				}
				//if exists attach files we do not gluing posts
				$gluing = false;
			}
		}
		
		
		// errors
		if ( !empty( $error ) ) {
			$_SESSION['addPostForm']            = array();
			$_SESSION['addPostForm']['error']   = '<p class="errorMsg">' . __('Some error in form') . '</p>'."\n".
			'<ul class="errorMsg">'."\n".$error.'</ul>'."\n";
			$_SESSION['addPostForm']['message'] = $message;
			redirect('/forum/view_theme/' . $id_theme);
		}

		
		$message = mb_substr($message, 0, Config::read('max_post_lenght', 'forum'));
		$id_user = $_SESSION['user']['id'];
		// Защита от того, чтобы один пользователь не добавил
		// 100 сообщений за одну минуту
		if ( isset( $_SESSION['unix_last_post'] ) and ( time() - $_SESSION['unix_last_post'] < 10 ) ) {
			return showInfoMessage(__('Your message has been added'), '/forum/view_theme/' . $id_theme );
		}
		
		
		//gluing posts
		if ($gluing === true) {
			$prev_post = $this->DB->select('posts', DB_FIRST, array(
				'cond' => array('id_theme' => $id_theme), 
				'order' => 'time DESC, id DESC'
			));
			if (count($prev_post) == 1) {
				$prev_post_author = $prev_post[0]['id_author'];
			}
			
			if (empty($prev_post_author)) $gluing = false;
			if ((mb_strlen($prev_post[0]['message'] . $message)) > Config::read('max_post_lenght', 'forum')) $gluing = false;
			if ($prev_post_author != $id_user) $gluing = false;
		}
		

		
		if ($gluing === true) {
			$message = $prev_post[0]['message'] . "\n\n" . '[color=939494]' . __('Added') . date("Y.m.d  H-i") . "[/color]\n\n" . $message;
			$this->DB->save('posts', array(
				'id' => $prev_post[0]['id'],
				'message' => $message,
				'time'      => new Expr('NOW()'),
			));
			
			$_res = $this->DB->save('themes', array(
				'last_post'      => new Expr('NOW()'),
				'id'             => $id_theme,
			));
			$this->DB->save('forums', array(
				'last_theme_id' => $id_theme,
				'id' => $res[0]['id_forum'],
			));
			
			
		} else {   // NOT GLUING MESSAGE

			// Все поля заполнены правильно - выполняем запрос к БД
			$_res = $this->DB->save('posts', array(
				'message'   => $message,
				'id_author' => $id_user,
				'time'      => new Expr('NOW()'),
				'id_theme'  => $id_theme
			));
			$post_id = mysql_insert_id();
			
			$attaches_exists = 0;
			// Массив недопустимых расширений файла вложения
			$extentions = array('.php', '.phtml', '.php3', '.html', '.htm', '.pl', '.PHP', '.PHTML', '.PHP3', '.HTML', '.HTM', '.PL');
			$img_extentions = array('.png','.jpg','.gif','.jpeg', '.PNG','.JPG','.GIF','.JPEG');
			/* delete collizions if exists */
			$this->deleteCollizions(array('id' => $post_id), true);
			for ($i = 1; $i < 6; $i++) {
				$attach_name = 'attach' . $i;
				if (!empty($_FILES[$attach_name]['name'])) {
					// Извлекаем из имени файла расширение
					$ext = strrchr($_FILES[$attach_name]['name'], ".");
					// Формируем путь к файлу
					if (in_array( $ext, $extentions)) {
						$file = $post_id . '-' . $i . '-' . date("YmdHi") . '.txt';
					} else {
						$file = $post_id . '-' . $i . '-' . date("YmdHi") . $ext;
					}	
					$is_image = 0;
					if ($_FILES[$attach_name]['type'] == 'image/jpeg'
					|| $_FILES[$attach_name]['type'] == 'image/jpg'
					|| $_FILES[$attach_name]['type'] == 'image/gif'
					|| $_FILES[$attach_name]['type'] == 'image/png') {
						$is_image = 1;
					}

					// Перемещаем файл из временной директории сервера в директорию files
					if (move_uploaded_file($_FILES[$attach_name]['tmp_name'], R . 'sys/files/forum/' . $file)) {
						chmod(R . 'sys/files/forum/' . $file, 0644);
						$attach_file_data = array(
							'post_id'       => $post_id,
							'theme_id'      => $id_theme,
							'user_id'       => $_SESSION['user']['id'],
							'attach_number' => $i,
							'filename'      => $file,
							'size'          => $_FILES[$attach_name]['size'],
							'date'          => new Expr('NOW()'),
							'is_image'      => $is_image,
						);
						if ($this->DB->save('forum_attaches', $attach_file_data)) {
							$attaches_exists = 1;
						}
					}
				}
			}
			if ($attaches_exists == 1) {
				$this->DB->save('posts', array(
					'id'       => $post_id,
					'attaches' => 1
				));
			}
			
			$cnt_posts_from_theme = $this->DB->select('posts', DB_COUNT, array('cond' => array('id_theme' => $id_theme)));
			$_res = $this->DB->save('themes', array(
				'id_last_author' => $id_user,
				'last_post'      => new Expr('NOW()'),
				'posts'          => ($cnt_posts_from_theme - 1),
				'id'             => $id_theme,
			));

			// speed spam protected
			$_SESSION['unix_last_post'] = time();

			// Обновляем количество сообщений для зарегистрированного пользователя
			if (isset($_SESSION['user'])) {
				$this->DB->save('users', array(
					'`posts` = `posts` + 1',
					'id' => $id_user
				));
			}
			//update forum info
			$this->DB->save('forums', array(
				'`posts` = `posts` + 1',
				'last_theme_id' => $id_theme,
				'id' => $res[0]['id_forum'],
			));
		}
		
		
		//clean cache
		$this->Cache->clean(CACHE_MATCHING_ANY_TAG, array('theme_id_' . $id_theme, 'user_id_' . $_SESSION['user']['id']));
		$this->DB->cleanSqlCache();
		
		
		if ($gluing === false) {
			if ($this->Log) $this->Log->write('adding post', 'post id(' . $post_id . '), theme id(' . $id_theme . ')');
			return showInfoMessage(__('Your message has been added'), '/forum/view_theme/' 
			. $id_theme . '?page=999#post' . $cnt_posts_from_theme );
		} else {
			if ($this->Log) $this->Log->write('adding post', 'post id(*gluing), theme id(' . $id_theme . ')');
			return showInfoMessage(__('Your message has been added'), '/forum/view_theme/' 
			. $id_theme . '?page=999#post' . $res[0]['posts'] );
		}
	}





	/**
	 * Create Edit post form
	 *
	 * @param int $id Post ID
	 */
	public function edit_post_form($id = null) {
		$id = (int)$id;
		if ($id < 1) redirect('/forum/');
		$writer_status = (!empty($_SESSION['user']['status'])) ? $_SESSION['user']['status'] : 0;


		// Получаем из БД сообщение
		$res = $this->DB->select('posts', DB_FIRST, array('cond' => array('id' => $id)));
		if (count($res) == 0 )
			return showInfoMessage(__('Some error occurred'),  '/forum/' );

		$post     = $res[0];
		$id_theme = $post['id_theme'];
		
		
		//check access
		if (!$this->ACL->turn(array('forum', 'edit_posts'), false) 
		&& (!empty($_SESSION['user']['id']) && $post['id_author'] == $_SESSION['user']['id'] 
		&& $this->ACL->turn(array('forum', 'edit_mine_posts'), false)) === false) {
			return showInfoMessage(__('Permission denied'), '/forum/');
		}


		$message = $post['message'];
		$html    = '';
		$markets = array();
		
		//if user vant preview message
		if (isset($_SESSION['viewMessage']) and !empty($_SESSION['viewMessage'])) {
			$view    = $this->Parser->getAndParse('previewmessage.html' );
			$view    = str_replace('{MESSAGE}', 
				$this->Textarier->print_page($_SESSION['viewMessage'], $writer_status), $view);
			$html    = $html . $view . "\n";
			$message = $_SESSION['viewMessage'];
			unset($_SESSION['viewMessage']);
		}

		// errors
		if (isset($_SESSION['editPostForm'])) {
			$info    = $this->Parser->getAndParse('infomessage.html');
			$info    = str_replace('{INFO_MESSAGE}', $_SESSION['editPostForm']['error'], $info);
			$html    = $info . $html . "\n";
			$message = $_SESSION['editPostForm']['message'];
			unset($_SESSION['editPostForm']);
		}

		
		$tpl = $this->Parser->getAndParse('editpostform.html' );
		$markets['{ACTION}'] = get_url('/forum/update_post/' . $id);
		$markets['{MAIN_TEXT}'] = h($message);
		
		
		/****  ATTACH  ****/
		$unlinkfiles = array(1 => '', 2 => '', 3 => '', 4 => '', 5 => '');
		if (!empty($post['attaches'])) {
			$attach_files = $this->DB->select('forum_attaches', DB_ALL, array('cond' => array('post_id' => $post['id'])));
			if (count($attach_files) > 0) {
				foreach ($attach_files as $attach_file) {
					if (file_exists(R . 'sys/files/forum/' . $attach_file['filename'])) {
						$unlinkfiles[$attach_file['attach_number']] = '<input type="checkbox" name="unlink' . $attach_file['attach_number'] 
						. '" value="1" />&nbsp;' . __('Delete') ."\n";
					}
				}
			}
		}
		foreach ($unlinkfiles as $key => $unlinkfile) {
			$markets['{ATTACH' . $key . '}'] = $unlinkfile;
		}
		/****  END  ATTACH  ****/
		
		
		// nav block
		$navi = array();
		$navi['{NAVIGATION}'] = get_link(__('Forums list'), '/forum/') . __('Separator')
		. get_link('Просмотр темы', '/forum/view_theme/' . $id_theme) . __('Separator') . __('Edit message');
		$this->_globalize($navi);
		
		
		$markets = $this->Parser->quoteTags($markets);
		$html = $html . $this->_replaceMarkets($markets, $tpl);
		return $this->_view($html);
	}



	/**
	 * Update Post record
	 *
	 * @param int $id Post ID
	 */
	public function update_post($id = null) {
		// Если не переданы данные формы - значит функция была вызвана по ошибке
		if (empty($id) || !isset($_POST['mainText'])) redirect('/forum/');
		$id = (int)$id;
		if ($id < 1) redirect('/forum/');


		// Проверяем, имеет ли пользователь право редактировать это сообщение (пост)
		$res = $this->DB->select('posts', DB_FIRST, array('cond' => array('id' => $id)));
		if (count($res) == 0 ) {
			return showInfoMessage(__('Some error occurred'), '/forum/' );
		}
		$post = $res[0];
		$id_theme = $post['id_theme'];
		
		//check access
		if (!$this->ACL->turn(array('forum', 'edit_posts'), false) 
		&& (!empty($_SESSION['user']['id']) && $post['id_author'] == $_SESSION['user']['id'] 
		&& $this->ACL->turn(array('forum', 'edit_mine_posts'), false)) === false) {
			return showInfoMessage(__('Permission denied'), '/forum/');
		}

		// Обрезаем сообщение до длины $set['forum']['max_post_lenght']
		$message = trim($_POST['mainText']);

		// Preview
		if (isset($_POST['viewMessage'])) {
			$_SESSION['viewMessage'] = $message;
			redirect('/forum/edit_post_form/' . $id);
		}

		// check fields...
		$error = '';
		if (empty($message))   
			$error = $error . '<li>' . __('Empty field "message"') . '</li>'."\n";
		if (mb_strlen($message) > Config::read('max_post_lenght', 'forum')) 
			$error = $error . '<li>' . sprintf(__('Very big message'), Config::read('max_post_lenght', 'forum')) . '</li>'."\n";
		// check attach 
		for ($i = 1; $i < 6; $i++) {
			if (!empty($_FILES['attach' . $i]['name'])) {
				if ($_FILES['attach' . $i]['size'] > Config::read('max_file_size')) {
					$error = $error . '<li>' . sprintf(__('Wery big file'), $i, (Config::read('max_file_size')/1024)) . '</li>'."\n";
				}
			}
		}
		
		
		/* if an error */
		if (!empty($error)) {
			$_SESSION['editPostForm']            = array();
			$_SESSION['editPostForm']['error']   = '<p class="errorMsg">' . __('Some error in form')
			. '</p>' . "\n" . '<ul class="errorMsg">'."\n".$error.'</ul>'."\n";
			$_SESSION['editPostForm']['message'] = $message;
			redirect('/forum/edit_post_form/' . $id);
		}
		
		
		
		/*****   ATTACH   *****/
		// Массив недопустимых расширений файла вложения
		$extentions = array('.php', '.phtml', '.php3', '.html', '.htm', '.pl', '.PHP', '.PHTML', '.PHP3', '.HTML', '.HTM', '.PL');
		$img_extentions = array('.png','.jpg','.gif','.jpeg', '.PNG','.JPG','.GIF','.JPEG');
		
		for ($i = 1; $i < 6; $i++) {
		
			if (!empty($_POST['unlink' . $i]) || !empty($_FILES['attach' . $i]['name'])) {
				$unlink_file = $this->DB->select('forum_attaches', DB_FIRST, array(
					'cond' => array(
						'post_id' => $id,
						'attach_number' => $i,
					)
				));
				/* may be collizions */
				if (count($unlink_file) > 1) $this->deleteCollizions(array('id' => $id), true);
				if (!empty($unlink_file) && file_exists(R . 'sys/files/forum/' . $unlink_file[0]['filename'])) {
					if(unlink(R . 'sys/files/forum/' . $unlink_file[0]['filename'])) {
						$this->DB->query("DELETE FROM `" . $this->DB->getFullTableName('forum_attaches') . "` 
						WHERE `post_id`='" . $id . "' AND `attach_number`='" . $i . "'");
					}
				}
			}
			
			$attach_name = 'attach' . $i;
			if (!empty($_FILES[$attach_name]['name'])) {
			
				// Извлекаем из имени файла расширение
				$ext = strrchr($_FILES[$attach_name]['name'], ".");
				// Формируем путь к файлу
				if (in_array( $ext, $extentions)) {
					$file = $id . '-' . $i . '-' . date("YmdHi") . '.txt';
				} else {
					$file = $id . '-' . $i . '-' . date("YmdHi") . $ext;
				}
				
				$is_image = 0;
				if ($_FILES[$attach_name]['type'] == 'image/jpeg'
				|| $_FILES[$attach_name]['type'] == 'image/jpg'
				|| $_FILES[$attach_name]['type'] == 'image/gif'
				|| $_FILES[$attach_name]['type'] == 'image/png') {
					$is_image = 1;
				}
				
				
				// Перемещаем файл из временной директории сервера в директорию files
				if (move_uploaded_file($_FILES[$attach_name]['tmp_name'], R . 'sys/files/forum/' . $file)) {
					chmod(R . 'sys/files/forum/' . $file, 0644);
					$attach_file_data = array(
						'post_id'       => $id,
						'theme_id'      => $id_theme,
						'user_id'       => $_SESSION['user']['id'],
						'attach_number' => $i,
						'filename'      => $file,
						'size'          => $_FILES[$attach_name]['size'],
						'date'          => new Expr('NOW()'),
						'is_image'      => $is_image,
					);
					$this->DB->save('forum_attaches', $attach_file_data);
				}
				
			}
		}
		$attach_exists = $this->DB->select('forum_attaches', DB_COUNT, array('cond' => array('post_id' => $id)));
		$attach_exists = ($attach_exists > 0) ? 1 : 0;
		/*****  END ATTACH   *****/

		
		
		// Все поля заполнены правильно - выполняем запрос к БД
		$message = mb_substr($message, 0, Config::read('max_post_lenght', 'forum'));
		$this->DB->save('posts', array(
			'message' => $message,
			'attaches' => $attach_exists,
			'id_editor' => $_SESSION['user']['id'],
			'edittime' => new Expr('NOW()'),
			'id' => $id
		));

		//clean cache
		$this->Cache->clean(CACHE_MATCHING_ANY_TAG, array('post_id_' . $id));
		$this->DB->cleanSqlCache();
		if ($this->Log) $this->Log->write('editing post', 'post id(' . $id . '), theme id(' . $id_theme . ')');
		return showInfoMessage(__('Operation is successful'), '/forum/view_theme/' . $id_theme . '?page=999' );

	}




	/**
	* deleting post from forum
	* @id     post ID
	* @return none
	*/
	public function delete_post($id = null) {
		$id = (int)$id;
		if (empty($id) || $id < 1) {
			redirect('/forum/');
		}

		// Получаем из БД информацию об удаляемом сообщении - это нужно,
		// чтобы узнать, имеет ли право пользователь удалить это сообщение
		$res = $this->DB->select('posts', DB_FIRST, array('cond' => array('id' => $id)));
		if (count($res) == 0) return showInfoMessage(__('Some error occurred'), '/forum/' );
		$post = $res[0];
		
		//check access
		if (!$this->ACL->turn(array('forum', 'delete_posts'), false) 
		&& (!empty($_SESSION['user']['id']) && $post['id_author'] == $_SESSION['user']['id'] 
		&& $this->ACL->turn(array('forum', 'delete_mine_posts'), false)) === false) {
			return showInfoMessage(__('Permission denied'), '/forum/');
		}

		$theme = $this->DB->select('themes', DB_FIRST, array('cond' => array('id' => $post['id_theme'])));

		// Удаляем файл, если он есть
		if (!empty($post['attaches'])) {
			$attach_files = $this->DB->select('forum_attaches', DB_ALL, array('cond' => array('post_id' => $id)));
			if (count($attach_files) > 0) {
				foreach ($attach_files as $attach_file) {
					if (file_exists(R . 'sys/files/forum/' . $attach_file['filename'])) {
						if (@unlink(R . 'sys/files/forum/' . $attach_file['filename'])) {
							$this->DB->query("DELETE FROM `" . $this->DB->getFullTableName('forum_attaches') . "` WHERE `id`='" . $attach_file['id'] . "'");
						}
					}
				}
			}
		}
		$this->DB->query("DELETE FROM `" . $this->DB->getFullTableName('posts') . "` WHERE `id` = '" . $id . "'");
		// Если это - единственное сообщение темы, то надо удалить и тему
		$res = $this->DB->select('posts', DB_COUNT, array('cond' => array('id_theme' => $post['id_theme'])));
		
		if ($res == 0) {
			// Прежде чем удалять тему, надо обновить таблицу TABLE_USERS
			$this->DB->query("UPDATE `" . $this->DB->getFullTableName('users') . "`
					  SET `themes` = `themes` - 1 WHERE `id` = (SELECT `id_author` 
					  FROM `" . $this->DB->getFullTableName('themes') . "` WHERE `id` = '" . $post['id_theme'] . "')");
			$this->DB->query("DELETE FROM `" . $this->DB->getFullTableName('themes') . "` WHERE `id` = '" . $post['id_theme'] . "'");
			// Если мы удалили тему, то мы не можем в нее вернуться;
			// поэтому редирект будет на страницу форума, а не страницу темы
			$deleteTheme = true;
		}
		
		
		// Обновляем количество сообщений, оставленных автором сообщения ...
		$this->DB->save('users', array(
			'`posts` = `posts` - 1',
			'id' => $post['id_author'],
		));
		// ... и таблицу .themes
		if (!isset($deleteTheme)) {
			$res = $this->DB->select('posts', DB_FIRST, array(
				'cond' => array('id_theme' => $post['id_theme']), 
				'fields' => array('id_author', 'time'), 
				'order' => 'id DESC',
			));
			$posts_cnt = $this->DB->select('posts', DB_COUNT, array('cond' => array('id_theme' => $post['id_theme'])));
			$id_last_author = $res[0]['id_author'];
			$last_post = $res[0]['time'];
			$this->DB->save('themes', array(
				'id_last_author' => $id_last_author,
				'last_post' => $last_post,
				'posts' => $posts_cnt - 1,
				'id' => $post['id_theme'],
			));
		}

		//clean cache
		$cahceKey = array('post_id_' . $id);
		if (isset($deleteTheme)) $cahceKey[] = 'theme_id_' . $post['id_theme'];
		$this->Cache->clean(CACHE_MATCHING_ANY_TAG, $cahceKey);
		$this->DB->cleanSqlCache();
		if ($this->Log) $this->Log->write('delete post', 'post id(' . $id . '), theme id(' . $post['id_theme'] . ')');
		
		
		//update forum info
		if (isset($deleteTheme)) {
			$this->DB->query("UPDATE `" . $this->DB->getFullTableName('forums') . "` SET `themes` = `themes` - 1, `posts` = `posts` - 1 
				, `last_theme_id`=(SELECT `id` FROM `" . $this->DB->getFullTableName('themes') . "` 
				WHERE `id_forum`='" . $theme[0]['id_forum'] . "' ORDER BY `last_post` DESC  LIMIT 1)
				WHERE `id` = '" . $theme[0]['id_forum'] . "'");
			if (count($theme) < 1) return showInfoMessage(__('Operation is successful'), '/forum/' );
			return showInfoMessage(__('Operation is successful'), '/forum/view_forum/' . $theme[0]['id_forum'] );
			
		} else {
			$this->DB->query("UPDATE `" . $this->DB->getFullTableName('forums') . "` SET `posts` = `posts` - 1 
				, `last_theme_id`=(SELECT `id` FROM `" . $this->DB->getFullTableName('themes') . "` 
				WHERE `id_forum`='" . $theme[0]['id_forum'] . "' ORDER BY `last_post` DESC  LIMIT 1)
				WHERE `id` = '" . $theme[0]['id_forum'] . "'");
			return showInfoMessage(__('Operation is successful'), '/forum/view_theme/' . $post['id_theme'] );
		}
	}



	
	/**
	 * View post for users
	 *
	 * @param ind $user_id
	 * @return none
	 */
	public function user_posts($user_id) {
	
		$this->page_title .= ' - ' . __('User messages');
		$html = '';
    
		if ($this->cached && $this->Cache->check($this->cacheKey)) {
			$html = $this->Cache->read($this->cacheKey);
			return $this->_view($html);
        }
		
		
		// Page nav
		$nav = array();
		$total = $this->DB->select('themes', DB_COUNT, array(
			'joins' => array(
				array(
					'table' => 'posts',
					'alias' => 'b',
					'cond' => '`b`.`id_theme` = `a`.`id`',
					'type' => 'LEFT',
				),
			),
			'fields' => array('DISTINCT(a.`id`)'),
			'alias' => 'a',
			'cond' => array("b.`id_author` = '" . $user_id . "'"),
		));
		$perPage = Config::read('themes_per_page', 'forum');

		$cntPages = ceil($total / $perPage);
		$page = (isset($_GET['page'])) ? (int)$_GET['page'] : 1;
		if ($page < 1) $page = 1;
		if ($page > $cntPages) $page = $cntPages;
		if ($cntPages > 1) {
			$nav['{PAGINATION}'] = pagination( $page, $cntPages, '/forum/user_posts/' . $user_id);
			$this->page_title .= ' (' . $page . ')';
		} else {
			$nav['{PAGINATION}'] = '&nbsp;';
		}
		
		$recOnPage = ($page == $cntPages) ? ($total % $perPage) : $perPage;
		$nav['{NAVIGATION}'] = get_link(__('Forums list'), '/forum/') . __('Separator') . __('User messages');
		$nav['{META}'] = __('Count all topics') . $total . '. ' . __('Count visible') . $recOnPage;
		$this->_globalize($nav);
		
		if ($total < 1) return $this->_view(__('No topics'));
		
		
		
		//get records
		$res = $this->DB->select('themes', DB_ALL, array(
			'joins' => array(
				array(
					'table' => 'posts',
					'alias' => 'e',
					'cond' => '`e`.`id_theme` = `a`.`id`',
					'type' => 'LEFT',
				),
				array(
					'table' => 'users',
					'alias' => 'b',
					'cond' => '`b`.`id` = `a`.`id_author`',
					'type' => 'LEFT',
				),
				array(
					'table' => 'users',
					'alias' => 'c',
					'cond' => '`c`.`id` = `a`.`id_last_author`',
					'type' => 'LEFT',
				),
				array(
					'table' => 'forums',
					'alias' => 'd',
					'cond' => '`d`.`id` = `a`.`id_forum`',
					'type' => 'LEFT',
				),
			),
			'cond' => array("e.`id_author` = '" . $user_id . "'"),
			'fields' => array('a.*', 'b.`name` AS author', 'c.`name` AS last_author', 'd.`title` AS forum_title'),
			'alias' => 'a',
			'order' => 'e.`time` DESC',
			'group' => 'a.`id`',
			'page' => $page,
			'limit' => Config::read('themes_per_page', 'forum'),
		));

		foreach ($res as $theme) {
			$markets = array(
				'{PARENT_FORUM}' => get_link($theme['forum_title']
				, '/forum/view_forum/' . $theme['id_forum']),
			);
			$template = $this->Parser->getAndParse('vievlastthemes.html');
			$row = $this->__parseThemeTable($theme, $template);
			$row = $this->_replaceMarkets($markets, $row);
			$html .= $row;

			
			
			//set cache tags
			$this->setCacheTag(array(
				'theme_id_' . $theme['id'],
			));
		}			
		
		
		$main_templ = $this->Parser->getAndParse('lastthemestable.html');
		$markets = array();
		$markets['{FORUM_NAME}'] = __('User messages');
		$markets['{THEMES}'] = $html;
		$html = $this->_replaceMarkets($markets, $main_templ);
		
		
		
		// write into cache
		if ($this->cached) {
			$this->Cache->write($html, $this->cacheKey, $this->cacheTags);
		}
		
		$this->_view($html);
	}
	

	
	/**
	* @return forum statistic block
	*/
	protected function _get_stat() {
		$html = $this->Parser->getAndParse('get_stat.html');
		$markets = array();
		
		
		$result = $this->DB->query("
			SELECT COUNT(*) as users_cnt
			, (SELECT `id` FROM `" . $this->DB->getFullTableName('users') . "` ORDER BY `puttime` DESC LIMIT 1) as last_user_id
			, (SELECT `name` FROM `" . $this->DB->getFullTableName('users') . "` ORDER BY `puttime` DESC LIMIT 1) as last_user_name
			, (SELECT COUNT(*) FROM `" . $this->DB->getFullTableName('posts') . "`) as posts_cnt
			, (SELECT COUNT(*) FROM `" . $this->DB->getFullTableName('themes') . "`) as themes_cnt
			FROM `" . $this->DB->getFullTableName('users') . "`");
		
		
		if (!empty($result[0]['last_user_id']) && !empty($result[0]['last_user_name'])) {
			$markets['{NEW_USER}'] = get_link(h($result[0]['last_user_name']), 
			'/users/info/' . $result[0]['last_user_id']);
		}
		$markets['{COUNT_USERS}'] = (!empty($result[0]['users_cnt'])) ? $result[0]['users_cnt'] : 0;
		$markets['{COUNT_POSTS}'] = (!empty($result[0]['posts_cnt'])) ? $result[0]['posts_cnt'] : 0;
		$markets['{COUNT_THEMES}'] = (!empty($result[0]['themes_cnt'])) ? $result[0]['themes_cnt'] : 0;

		
		// Пользователи on-line
		$path = R . 'sys/logs/counter_online/online.dat';
		if (file_exists($path) && is_readable($path)) {
			$data = unserialize(file_get_contents($path));
			

			$markets['{ONLINE_USERS}'] = '';
			$users_links = array();
			if (count($data['users']) > 0 && is_array($data['users'])) {
				foreach ($data['users'] as $id => $info) {
					if (strstr($id, 'bot')) {
						$users_links[] = '<span class="botname">' . $info['name'] . '</span>';
						continue;
					}
				
					$color = '';
					if (isset($info['status'])) {
						$group_info = $this->ACL->get_user_group($info['status']);
						if (!empty($group_info['color'])) $color = 'style="color:#' . $group_info['color'] . '"';
					}
					$users_links[] = '<a ' . $color . ' href="' . 
					get_url('/users/info/' . $id) . '">' . h($info['name']) . '</a>';
				}
				$markets['{ONLINE_USERS}'] = implode($users_links, ', ');
			}

		} else {
			$markets['{ONLINE_USERS}'] = '';
		}
		
		
		$html = $this->_replaceMarkets($markets, $html);
		return $html;
	}



	
	public function download_file($file = null, $mimetype = 'application/octet-stream') {
		if (empty($file)) redirect('/');
		
		$path = R . 'sys/files/forum/' . $file;
		if (!file_exists($path)) die(__('File not found'));
		$from=0;
		$size=filesize($path);
		$to=$size;
		if (isset($_SERVER['HTTP_RANGE'])) {
			if (preg_match ('#bytes=-([0-9]*)#',$_SERVER['HTTP_RANGE'],$range)) {// если указан отрезок от конца файла
				$from=$size-$range[1];
				$to=$size;
			} elseif(preg_match('#bytes=([0-9]*)-#',$_SERVER['HTTP_RANGE'],$range)) {// если указана только начальная метка

				$from=$range[1];
				$to=$size;
			} elseif(preg_match('#bytes=([0-9]*)-([0-9]*)#',$_SERVER['HTTP_RANGE'],$range)) {// если указан отрезок файла

				$from=$range[1];
				$to=$range[2];
			}
			header('HTTP/1.1 206 Partial Content');

			$cr='Content-Range: bytes '.$from .'-'.$to.'/'.$size;
		} else
			header('HTTP/1.1 200 Ok');
		
		$etag = md5($path);
		$etag = substr($etag, 0, 8) . '-' . substr($etag, 8, 7) . '-' . substr($etag, 15, 8);
		header('ETag: "'.$etag.'"');
		header('Accept-Ranges: bytes');
		header('Content-Length: ' .($to-$from));
		if (isset($cr)) header($cr);
		header('Connection: close');	
		header('Content-Type: ' . $mimetype);
		header('Last-Modified: ' . gmdate('r', filemtime($path)));
		header("Last-Modified: ".gmdate("D, d M Y H:i:s", filemtime($path))." GMT");
		header("Expires: ".gmdate("D, d M Y H:i:s", time() + 3600)." GMT");
		$f=fopen($path, 'rb');


		if (preg_match('#^image/#',$mimetype))
			header('Content-Disposition: filename="'.$file.'";');
		else
			header('Content-Disposition: attachment; filename="'.$file.'";');

		fseek($f, $from, SEEK_SET);
		$size=$to;
		$downloaded=0;
		while(!feof($f) and ($downloaded<$size)) {
			$block = min(1024*8, $size - $downloaded);
			echo fread($f, $block);
			$downloaded += $block;
			flush();
		}
		fclose($f);
	}

	
	public function important($id = null) {
		//turn access
		$this->ACL->turn(array('forum', 'important_themes'));
		$id = (int)$id;
		if ($id < 1) {
			redirect('/forum/');
		}
		
		
		$theme = $this->DB->select('themes', DB_FIRST, array('cond' => array('id' => $id)));
		if (count($theme) < 1) return showInfoMessage(__('Some error occurred'), '/forum/');
		
		$this->DB->save('themes', array(
			'important' => '1',
			'id' => $id,
		));
		/* clean cache DB*/
		$this->DB->cleanSqlCache();
		if ($this->Log) $this->Log->write('important post', 'post id(' . $id . '), theme id(' . $theme[0]['id'] . ')');
		return showInfoMessage(__('Operation is successful'), '/forum/view_forum/' . $theme[0]['id_forum']);
	}
	
	
	public function unimportant($id = null) {
		//turn access
		$this->ACL->turn(array('forum', 'important_themes'));
		$id = (int)$id;
		if ($id < 1) {
			redirect('/forum/');
		}
		
		$theme = $this->DB->select('themes', DB_FIRST, array('cond' => array('id' => $id)));
		if (count($theme) < 1) return showInfoMessage(__('Some error occurred'), '/forum/');
		
		$this->DB->save('themes', array(
			'important' => '0',
			'id' => $id
		));

		/* clean cache DB */
		$this->DB->cleanSqlCache();
		if ($this->Log) $this->Log->write('unimportant post', 'post id(' . $id . '), theme id(' . $theme[0]['id'] . ')');
		return showInfoMessage(__('Operation is successful'), '/forum/view_forum/' . $theme[0]['id_forum']);
	}
	
	
	
	/**
	* deleting attaches  collizion 
	*
	* @post (array)   reply data
	* @clean(boolean) clean all or only collizions
	* @return         none
	*/
	private function deleteCollizions($post, $clean = false) {
		if (!is_array($post)) return;
		/* DB has file */
		if ($clean === true) {
			$this->DB->query("DELETE FROM `" . $this->DB->getFullTableName('forum_attaches') . "` WHERE `post_id`='" . $post['id'] . "'");
		} else {
			$attachments = $this->DB->select('forum_attaches', DB_ALL, array('cond' => array('post_id' => $post['id'])));
			if (!empty($attachments)) {
				foreach ($attachments as $key => $attach) {
					if (file_exists(R . 'sys/files/forum/' . $attach['filename'])) {
						clearstatcache();
						continue;
					}
					$this->DB->query("DELETE FROM `" . $this->DB->getFullTableName('forum_attaches') . "` WHERE `id`='" . $attach['id'] . "'");
					unset($attachments[$key]);
				}
			}
		}
		/* File has DB record */
		$attach_files = glob(R . 'sys/files/forum/' . $post['id'] . '-*');
		if (!empty($attach_files)) {
			foreach ($attach_files as $_key => $attach_file) {
				if ($clean === true) {
					@unlink($attach_file);
					continue;
				}
				$record_exists = false;
				if (!empty($attachments)) {
					foreach ($attachments as $attach) {
						if (strrchr($attach_file, '/') == $attach['filename']) {
							$record_exists = true;
							break;
						}
					}
				}
				if ($record_exists === false) {
					unset($attach_files[$_key]);
					@unlink($attach_file);
				}
			}
		}
		if ($clean === true) return;
		/* posts.attaches flag */
		$flag = (!empty($attach_files) && !empty($attachments)) ? 1 : 0;
		if ($flag != $post['attaches']) {
			$this->DB->save('posts', array('id' => $post['id'], 'attaches' => $flag));
		}
		return;
	}



	//delete theme
	private function __delete_theme($id_theme) {

		//we must know id_forum
		$theme = $this->DB->select('themes', DB_FIRST, array('cond' => array('id' => $id_theme)));
	
		// delete colision ( this is paranoia )
		$this->DB->query("DELETE FROM `" . $this->DB->getFullTableName('themes') . "` WHERE id NOT IN (SELECT DISTINCT id_theme FROM `posts`)");
		$this->DB->query("DELETE FROM `" . $this->DB->getFullTableName('posts') . "` WHERE id_theme NOT IN (SELECT id FROM `themes`)");

		// Сперва мы должны удалить все сообщения (посты) темы;
		// начнем с того, что удалим файлы вложений
		$res = $this->DB->select('posts', DB_ALL, array('cond' => array('id_theme' => $id_theme)));
		if (count($res) > 0) {
			foreach ($res as $file) {
				// Удаляем файл, если он есть
				$attach_files = $this->DB->select('forum_attaches', DB_ALL, array('cond' => array('post_id' => $file['id'])));
				if (count($attach_files) > 0) {
					foreach ($attach_files as $attach_file) {
						if (file_exists(R . 'sys/files/forum/' . $attach_file['filename'])) {
							if (@unlink(R . 'sys/files/forum/' . $attach_file['filename'])) {
								$this->DB->query("DELETE FROM `" . $this->DB->getFullTableName('forum_attaches') . "` WHERE `id`='" . $attach_file['id'] . "'");
							}
						}
					}
				}
				// заодно обновляем таблицу USERS
				if ($file['id_author']) {
					$this->DB->save('users', array(
						'`posts` = `posts` - 1',
						'id' => $file['id_author'],
					));
				}
			}
		}
		
		
		$attach_files = $this->DB->select('forum_attaches', DB_ALL, array('cond' => array('theme_id' => $id_theme)));
		if (count($attach_files) > 0) {
			foreach ($attach_files as $attach_file) {
				if (file_exists(R . 'sys/files/forum/' . $attach_file['filename'])) {
					if (@unlink(R . 'sys/files/forum/' . $attach_file['filename'])) {
						$this->DB->query("DELETE FROM `" . $this->DB->getFullTableName('forum_attaches') . "` WHERE `id`='" . $attach_file['id'] . "'");
					}
				}
			}
		}
		
		
		if (count($theme) > 0) {
			// Обновляем таблицу TABLE_USERS - надо обновить поле themes
			$u_res = $this->DB->query(
				"UPDATE `" . $this->DB->getFullTableName('users') . "` SET 
				`themes` = (SELECT COUNT(*) FROM `" . $this->DB->getFullTableName('themes') . "` 
				WHERE `id_author` = '" . $theme[0]['id_author'] . "')
				, `posts` = (SELECT COUNT(*) FROM `" . $this->DB->getFullTableName('posts') . "` 
				WHERE `id_author` = '" . $theme[0]['id_author'] . "')
				WHERE `id` = '" . $theme[0]['id_author'] . "'");

			//update forum info
			$this->DB->query(
				"UPDATE `" . $this->DB->getFullTableName('forums') . "` SET `themes` = 
				(SELECT COUNT(*) FROM `" . $this->DB->getFullTableName('themes') . "` 
				WHERE `id_forum` = '" . $theme[0]['id_forum'] . "'), `posts` = 
				(SELECT COUNT(b.`id`) FROM `" . $this->DB->getFullTableName('themes') . "` a 
				LEFT JOIN `" . $this->DB->getFullTableName('posts') . "` b ON a.`id`=b.`id_theme`),
				`last_theme_id`=(SELECT `id` FROM `" . $this->DB->getFullTableName('themes') . "` 
				WHERE `id_forum`='" . $theme[0]['id_forum'] . "'
				ORDER BY `last_post` DESC  LIMIT 1) WHERE `id` = '" . $theme[0]['id_forum'] . "'" );
		}
		//clean cache
		$this->Cache->clean(CACHE_MATCHING_ANY_TAG, array('theme_id_' . $id_theme,));
		$this->Cache->clean(CACHE_MATCHING_TAG, array('module_forum', 'action_index'));
		$this->DB->cleanSqlCache();
		if ($this->Log) $this->Log->write('delete theme(because error uccured)', 'theme id(' . $id_theme . ')');
	}	
	
}
?>