<?php
/*---------------------------------------------\
|											   |
| Author:       Andrey Brykin (Drunya)         |
| Version:      1.5                            |
| Project:      CMS                            |
| package       CMS Fapos                      |
| subpackege    Helpers library                |
| copyright     ©Andrey Brykin 2010-2011       |
|----------------------------------------------|
|											   |
| any partial or not partial extension         |
| CMS Fapos,without the consent of the         |
| author, is illegal                           |
|----------------------------------------------|
| Любое распространение                        |
| CMS Fapos или ее частей,                     |
| без согласия автора, является не законным    |
\---------------------------------------------*/



/**
 * CRON simulyation
 */
function fpsCron($func, $interval) {
	$cron_file = R . 'sys/tmp/' . md5($func) . '_cron.dat';
	if (file_exists($cron_file)) {
		$extime = file_get_contents($cron_file);
		if (!empty($extime) && is_numeric($extime) && $extime > time()) {
			return;
		}
	}

	if (function_exists($func)) {
		call_user_func($func);
		file_put_contents($cron_file, (time() + intval($interval)));
	}
}
 
 
 
 
/**
 * Launch auto sitemap generator
 */
function createSitemap() {
	include_once R . 'sys/inc/sitemap.class.php';
	$obj = new FpsSitemapGen;
	$obj->createMap();
}
 
 


/**
 * Only create HLU URL by title
 *
 * @param stirng title
 * @return string
 */
function getHluUrlByTitle($title) {
	$title = translit($title);
	$title = strtolower(preg_replace('#[^a-z0-9]#i', '_', $title));
	$hlu_extention = Config::read('hlu_extention');
	return $title . $hlu_extention;
}



/**
 * Create human like URL.
 * Get title of material and create url
 * from this title. OR create simple URL, if hlu is off.
 * 
 * @param array $materila
 * @param string $module
 * @return string 
 */
function entryUrl($material, $module) {
	if (empty($material['id'])) 
		trigger_error('Empty material ID', E_USER_ERROR);
		
	if (Config::read('hlu') != 1 || empty($material['title'])) {
		$url = $module . '/view/' . $material['id'];
		return $url;
	}
	
	// extention
	$extention = '';
	$hlu_extention = Config::read('hlu_extention');
	if (!empty($hlu_extention)) {
		$extention = $hlu_extention;
	}
	
	// URL pattern
	$pattern = '/' . $module . '/%s' . $extention;
	
	
	// Check tmp file with assocciations and build human like URL
	clearstatcache();
	$tmp_dir = R . 'sys/tmp/hlu_' . $module . '/';
	$tmp_file = $tmp_dir . $material['id'] . '.dat';
	touchDir($tmp_dir, 0777);
	if (file_exists($tmp_file) && is_readable($tmp_file)) {
		$title = file_get_contents($tmp_file);
		if (!empty($title)) {			
		
			if (!file_exists($tmp_dir . $title . '.dat')) {
				file_put_contents($tmp_dir . $title . '.dat', $materila['id']);
			}
			return h(sprintf($pattern, $title));
		}
	}
	
	
	$title = translit($material['title']);
	$title = strtolower(preg_replace('#[^a-z0-9]#i', '_', $title));
	
	// Colission protect
	$tmp_file_title = $tmp_dir . $title . '.dat';
	while (file_exists($tmp_file_title)) {
		$collision = file_get_contents($tmp_file_title);
		if (!empty($collision) && $collision != $material['id']) {
			$title .= '_';
			$tmp_file_title = $tmp_dir . $title . '.dat';
		}
	}

	
	file_put_contents($tmp_file, $title);
	file_put_contents($tmp_dir . $title . '.dat', $material['id']);
	return h(sprintf($pattern, $title));
}



/**
 * Translit. Convert cirilic chars to 
 * latinic chars.
 *
 * @param string $str
 * @return string 
 */
function translit($str) {
	$cirilic = array('й', 'ц', 'у', 'к', 'е', 'н', 'г', 'ш', 'щ', 'з', 'х', 'ъ', 'ф', 'ы', 'в', 'а', 'п', 'р', 'о', 'л', 'д', 'ж', 'э', 'я', 'ч', 'с', 'м', 'и', 'т', 'ь', 'б', 'ю', 'ё', 'Й', 'Ц', 'У', 'К', 'Е', 'Н', 'Г', 'Ш', 'Щ', 'З', 'Х', 'Ъ', 'Ф', 'Ы', 'В', 'А', 'П', 'Р', 'О', 'Л', 'Д', 'Ж', 'Э', 'Я', 'Ч', 'С', 'М', 'И', 'Т', 'Ь', 'Б', 'Ю', 'Ё');
	$latinic = array('i', 'c', 'u', 'k', 'e', 'n', 'g', 'sh', 'sh', 'z', 'h', '', 'f', 'y', 'v', 'a', 'p', 'r', 'o', 'l', 'd', 'j', 'e', 'ya', 'ch', 's', 'm', 'i', 't', '', 'b', 'yu', 'yo', 'i', 'c', 'u', 'k', 'e', 'n', 'g', 'sh', 'sh', 'z', 'h', '', 'f', 'y', 'v', 'a', 'p', 'r', 'o', 'l', 'd', 'j', 'e', 'ya', 'ch', 's', 'm', 'i', 't', '', 'b', 'yu', 'yo');
	
	return str_replace($cirilic, $latinic, $str);
}




/**
 * Create captcha input field and image with 
 * security code.
 *
 */
function getCaptcha() {
	$kcaptcha = '/sys/inc/kcaptcha/kc.php?' . session_name() . '=' . session_id();
	$tpl = file_get_contents(R . 'template/' . Config::read('template') . '/html/default/captcha.html');
	return str_replace('{CAPTCHA}', $kcaptcha, $tpl);
}



/**
 * Work for language pack.
 * Open language file and return needed
 * string.
 *
 * @param int $key
 * @param string $module
 * @return string
 */
function __($key, $module = false) {
	
	if (is_numeric($key)) {
		if ($module === false) $module = 'common';
		
		$lan_dir = R . 'sys/settings/languages/' . $module . '.dat';
		$data = file_get_contents($lan_dir);
		if (empty($data)) trigger_error('Language file not found!', E_USER_ERROR);
		$data = explode("=|=", $data);
		return (!empty($data[$key])) ? trim($data[$key]) : '';
		
	} else {
		$lang_file = R . 'sys/settings/languages/russian.php';
		if (!file_exists($lang_file)) trigger_error('Language file not found', E_USER_ERROR);
		
		include $lang_file;
		if (array_key_exists($key, $language)) return $language[$key];
		return $key;
	}
}




/**
 * Uses for valid create HTML tag IMG
 * and fill into him correctli url.
 * When you use this function you
 * mustn't wory obout Fapos install
 * into subdri or SUBDIRS.
 * ALso if we wont change class of IMG or etc,
 * we change this only here and this changes apply
 * for evrywhere.
 *
 * @param string $url
 * @param array $params
 * @param boolean $notRoot
 * @return string HTML link
 */
function get_img($url, $params = array(), $notRoot = false) {
	$additional = '';
	if (!empty($params) && is_array($params)) {
		foreach($params as $key => $value) {
			$additional .= h($key) . '="' . h($value) . '" ';
		}
	}
	return '<img  ' . $additional . 'src="' . get_url($url, $notRoot) . '" />';
}


/**
 * Uses for valid create url.
 * When you use this function you
 * mustn't wory obout Fapos install
 * into subdri or SUBDIRS.
 * This function return url only from root (/)
 * But you can send $notRoot and get url from not root.
 *
 * @param string $url
 * @param boolean $notRoot
 * @return string url
 */
function get_url($url, $notRoot = false) {
	if ($notRoot) return $url;
	$url = '/' . WWW_ROOT . $url;
	// May be collizions
	return str_replace('//', '/', $url);
}



/**
 * Uses for valid create HTML tag A
 * and fill into him correctli url.
 * When you use this function you
 * mustn't wory obout Fapos install
 * into subdri or SUBDIRS.
 * ALso if we wont change class of A or etc,
 * we change this only here and this changes apply
 * for evrywhere.
 *
 * @param string $ankor
 * @param string $url
 * @param array $params
 * @param boolean $notRoot
 * @return string HTML link
 */
function get_link($ankor, $url, $params = array(), $notRoot = false) {
	$additional = '';
	if (!empty($params) && is_array($params)) {
		foreach($params as $key => $value) {
			$additional .= h($key) . '="' . h($value) . '" ';
		}
	}
	$link = '<a ' . $additional . 'href="' . get_url($url, $notRoot) . '">' . $ankor . '</a>';
	return $link;
}


/**
 * doing hard redirect
 * Send header and if header do not
 * work stop script and die. Better redirect
 * user to another page but if can't doing this
 * better stop script.
 */
function redirect($url) {
	header('Location: ' . get_url($url));
	die() or exit();
}



/*
* return who online
* also we have analog in statistic module
*/
function getWhoOnline() {
	$path = R . 'sys/logs/counter_online/online.dat';
	$users = 0;
	$quests = 0;
	$all = 0;
	
	if (file_exists($path) && is_readable($path)) {
		$data = unserialize(file_get_contents($path));
		$users = count($data['users']);
		$quests = count($data['guests']);
		$all = ($quests + $users);
	}
	
	return array('users' => $users, 'guests' => $quests, 'all' => $all);
}


/**
 * Return online users list
 */
function getOnlineUsers() {
	$path = R . 'sys/logs/counter_online/online.dat';
	
	if (file_exists($path) && is_readable($path)) {
		$data = unserialize(file_get_contents($path));
		$users = $data['users'];
	}
	
	return $users;
}


/**
* touch and create dir
*/
function touchDir($path, $chmod = 0777) {
	if (!file_exists($path)) {
		mkdir($path, $chmod, true);
		chmod($path, $chmod);
	}
	return true;
}


/**
* print visibility param value
* @param string or array
*/
function pr($param) {
	echo '<pre>' . print_r($param, true) . '</pre>';
}


/**
* short version "htmlspecialchars()"
* @param string or array
*/
function h($param) {

	if (!is_array($param)) {
		$param = htmlspecialchars($param);
		$symbols = array(
			'&#125;' => '&amp;#125;',
			'&#123;' => '&amp;#123;',
		);
		return str_replace($symbols, array_keys($symbols), $param);
	}
	
	if (is_array($param)) {
		foreach ($param as $key => $value) {
			$param[$key] = h($value);
		}
		
		return $param;
	}
	
	return false;
}


/**
* @return timestamp on microseconds
*/
function getMicroTime() { 
    list($usec, $sec) = explode(" ", microtime()); 
    return ((float)$usec + (float)$sec); 
} 


/**
* for tests and dumps
*/
function dumpVar($var) {
	$f = fopen(R . 'dump.dat', 'a+');
	fwrite($f, $var . "\n");
	fclose($f);
}


/**
 * mysql_real_escape_string copy
 */
function resc($str) {
	return mysql_real_escape_string($str);
}
?>