<?php
/*-----------------------------------------------\
| 												 |
|  Author:       Andrey Brykin (Drunya)          |
|  Version:      0.2                             |
|  Project:      CMS                             |
|  package       CMS Fapos                       |
|  subpackege    Additional Fields               |
|  copyright     ©Andrey Brykin 2010-2011        |
\-----------------------------------------------*/

/*-----------------------------------------------\
| 												 |
|  any partial or not partial extension          |
|  CMS Fapos,without the consent of the          |
|  author, is illegal                            |
|------------------------------------------------|
|  Любое распространение                         |
|  CMS Fapos или ее частей,                      |
|  без согласия автора, является не законным     |
\-----------------------------------------------*/


/**
 * Begin work when additional fields mode is ON
 *
 * @author     Andrey Brykin
 * @version    0.2
 * @url        http://fapos.net
 */
class FpsAdditionalFields {
 
	
	public $module;
	
	
	public function __construct() {
		$this->DB = FpsDataBase::get();
	}
	
	
	/**
	 * Merge result of SQL query without additional fields with
	 * additional fields.
	 */
	public function mergeRecords($records, $inputs = false, $module = null) {
		if (empty($module)) $module = $this->module;
		$newsIds = array();
		foreach ($records as $record) 
			$newsIds[] = $record['id'];
		$newsIds = implode(', ', $newsIds);
		
		$addFields = $this->DB->select($module . '_add_fields', DB_ALL, array(
			'joins' => array(
				array(
					'table' => $module . '_add_content',
					'alias' => 'b',
					'type' => 'LEFT',
					'cond' => array(
						'`a`.`id` = `b`.`field_id`',
						'`b`.`entity_id` IN (' . $newsIds . ')',
					),
				),
			),
			'alias' => 'a',
			'fields' => array('`a`.*', '`b`.`content`', '`b`.`entity_id`', '`b`.`field_id`'),
		));
		
		if (count($addFields) > 0) {
			foreach ($records as $k => $n) {
				foreach ($addFields as $f) {
					if($n['id'] != $f['entity_id']) {
						unset($f['content']);
					}
					$field = 'add_field_' . $f['id'];
					$f_params = (!empty($f['params'])) ? unserialize($f['params']) : array();
					

					if ($inputs === true) {
						if (isset($_SESSION['viewMessage'][$field])) 
							$f['content'] = h($_SESSION['viewMessage'][$field]);
						if (isset($_SESSION['FpsForm'][$field])) 
							$f['content'] = h($_SESSION['FpsForm'][$field]);
					}
					
					switch ($f['type']) {
						case 'textarea':// TEXTAREA
							$records[$k][$field] = (!empty($f['content'])) ? h($f['content']) : '';
							if ($inputs === true) {
								$records[$k][$field] = '<textarea name="' . $field . '">'
								. $records[$k][$field] . '</textarea>';
							}
							break;
							
							
						case 'text':// TEXT
							$records[$k][$field] = (!empty($f['content'])) ? h($f['content']) : '';
							if ($inputs === true) {
								$records[$k][$field] = '<input type="text" name="' . $field . '"'
								. ' value="' . $records[$k][$field] . '" />';
							}
							break;
							
							
						case 'checkbox':// CHECKBOX
							$ans = (!empty($f_params['values'])) ? explode('|', $f_params['values']) : array();
							$yes = (!empty($ans[0])) ? h($ans[0]) : '';
							$no = (!empty($ans[1])) ? h($ans[1]) : '';
							$records[$k][$field] = (!empty($f['content'])) ? $yes : $no;
							if ($inputs === true) {
								if (empty($f['content']))
								$records[$k][$field] = '<input type="checkbox" name="' . $field . '" value="1" />';
								else
								$records[$k][$field] = '<input type="checkbox" name="' . $field . '"'
								. 'value="1" checked="checked" />';
							}
							break;
							
							
						default:
							$records[$k][$field] = (!empty($f['content'])) ? h($f['content']) : '';
							if ($inputs === true) {
								$records[$k][$field] = '<input type="text" name="' . $field . '"'
								. ' value="' . $records[$k][$field] . '" />';
							}
							break;
					}
				}
			}
		}
		return $records;
	}


	
	/**
	 *
	 */
	public function getInputs($records = array(), $setValues = true, $module = null) {
		if (empty($module)) $module = $this->module;
		if (!empty($records)) $addFields = $records;
		else $addFields = $this->DB->select($module . '_add_fields', DB_ALL);
		$_addFields = array();
		$output = array();
		
		
		if (!empty($addFields)) {
			foreach($addFields as $key => $field) {
				$_addFields[$key] = 'add_field_' . $field['id'];
				$output[$_addFields[$key]] = '';
				$value = '';
				
				if ($setValues) {
					if (!empty($_SESSION['viewMessage'][$_addFields[$key]])) 
						$value = h($_SESSION['viewMessage'][$_addFields[$key]]);
					if (!empty($_SESSION['FpsForm'][$_addFields[$key]])) 
						$value = h($_SESSION['FpsForm'][$_addFields[$key]]);
				}

				switch ($field['type']) {
					case 'text':
						$output[$_addFields[$key]] = '<input type="text" value="' . $value 
						. '" name="' . $_addFields[$key] . '" />';
						break;
					case 'textarea':
						$output[$_addFields[$key]] = '<textarea name="' . $_addFields[$key] . '">' 
						. $value . '</textarea>';
						break;
					case 'checkbox':
						$output[$_addFields[$key]] = '<input type="checkbox" value="1" name="' 
						. $_addFields[$key] . '" ';
						$output[$_addFields[$key]] .= (!empty($value)) ? 'checked="checked" />' : ' />';
						break;
				}
			}
		}
		return $output;
	}

	
	
	/**
	 *
	 */
	public function checkFields($module = null) {
		if (empty($module)) $module = $this->module;
		$addFields = $this->DB->select($module . '_add_fields', DB_ALL);
		$_addFields = array();
		$error = null;
		
		
		if (!empty($addFields)) {
			foreach($addFields as $key => $field) {
				$params = (!empty($field['params'])) 
				? unserialize($field['params']) : array();
				$field_name = 'add_field_' . $field['id'];
				$value = '';
				$array = array('field_id' => $field['id']);
				
				
				
				
				switch ($field['type']) {
					case 'textarea':
					case 'text':
						if (!empty($params['required']) && empty($_POST[$field_name])) 
							$error = $error . '<li>Поле "' . $field['label'] . '" не заполнено</li>';
						if (!empty($_POST[$field_name]) && mb_strlen($_POST[$field_name]) > (int)$field['size']) 
							$error = $error . '<li>Поле "' . $field['label'] . '" превышает ' 
							. $field['size'] . ' символов</li>';
						$array['content'] = (!empty($_POST[$field_name])) ? $_POST[$field_name] : '';
						break;
					case 'checkbox':
						if (!empty($_POST[$field_name])) 
							$array['content'] = 1;
						else
							$array['content'] = 0;
						break;
				}
				$_addFields[] = $array;
			}
		}
		return (!empty($error)) ? $error : $_addFields;
	}
	
	
	
	/**
	 * For saving additional fields
	 */
	public function save($rec_id, $fields, $module = null) {
		if (empty($module)) $module = $this->module;
	
		if (empty($fields)) return true;
		foreach ($fields as $field) {
			$check = $this->DB->select($module . '_add_content', DB_FIRST, array(
				'cond' => array(
					'entity_id' => $rec_id,
					'field_id' => $field['field_id'],
				),
			));
			$conditions = (count($check) < 1) ? array() : array('id' => $check[0]['id']); 
			$this->DB->save($module . '_add_content', array(
				'entity_id' => $rec_id,
				'field_id' => $field['field_id'],
				'content' => $field['content'],
			), $conditions);
		}
	}
	
	
	
	
}
?>